# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_perc_rule_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def mme_explanation(input_parameters):
    explanation = r"""
    The Opioid Conversion Table with MME (Morphine Milligram Equivalent) 
    conversion factors are listed below:
       1. Codeine: MME conversion factor = 0.15
       2. FentaNYL buccal: MME conversion factor = 0.13
       3. HYDROcodone: MME conversion factor = 1
       4. HYDROmorphone: MME conversion factor = 5
       5. Methadone: MME conversion factor = 4.7
       6. Morphine: MME conversion factor = 1
       7. OxyCODONE: MME conversion factor = 1.5
       8. OxyMORphone: MME conversion factor = 3
       9. Tapentadol: MME conversion factor = 0.4
       10. TraMADol: MME conversion factor = 0.2
       11. Buprenorphine: MME conversion factor = 10
    """

    explanation += (
        "\n\nThe curent Morphine Milligram Equivalents "
        "(MME) is 0 MME per day.\n"
    )

    mme_drug = {
        "Codeine": 0.15,
        "FentaNYL buccal": 0.13,
        "FentANYL patch": 2.4,
        "HYDROcodone": 1,
        "HYDROmorphone": 5,
        "Methadone": 4.7,
        "Morphine": 1,
        "OxyCODONE": 1.5,
        "OxyMORphone": 3,
        "Tapentadol": 0.4,
        "TraMADol": 0.2,
        "Buprenorphine": 10,
    }

    mme_equivalent = 0

    for drug_name in input_parameters:
        if "Day" in drug_name:
            continue

        name = drug_name.split(" Dose")[0]

        units = input_parameters[name + " Dose"][1]

        if name != "FentaNYL buccal" and name != "FentaNYL patch":
            drug_mg_exp, drug_mg = conversion_explanation(
                input_parameters[name + " Dose"][0],
                name,
                None,
                None,
                units,
                "mg",
            )
            if units == "mg":
                explanation += (
                    f"The patient's dose of {name} is {drug_mg} " f"mg. "
                )
            else:
                explanation += (
                    f"The patient's dose of {name} is measured "
                    f"in {units}. We need to convert this to mg. "
                )
                explanation += drug_mg_exp + "\n"
        else:
            drug_mg_exp, drug_mg = conversion_explanation(
                input_parameters[name + " Dose"][0],
                name,
                None,
                None,
                units,
                "µg",
            )
            if units == "µg":
                explanation += (
                    f"The patient's dose of {name} is {drug_mg} " f"µg.\n"
                )
            else:
                explanation += (
                    f"The patient's dose of {name} is measured "
                    f"in {units}. We need to convert this to µg. "
                )
                explanation += drug_mg_exp + "\n"

        target_unit = (
            "mg"
            if name != "FentaNYL buccal" and name != "FentaNYL patch"
            else "µg"
        )

        dose_per_day_key = name + " Dose Per Day"

        dose_per_day = input_parameters[dose_per_day_key][0]

        total_per_day = round_number(drug_mg * dose_per_day)

        explanation += (
            f"The patient takes {dose_per_day} doses/day of "
            f"{name}. This means that the patient takes "
            f"{round_number(drug_mg)} {target_unit}/dose "
            f"{name} * {dose_per_day} dose/day = "
            f"{total_per_day} {target_unit}/day. "
        )

        explanation += (
            f"To convert to mme/day of {name}, multiply the "
            f"{total_per_day} {target_unit}/day by the mme "
            f"conversion factor, {mme_drug[name]} mme/"
            f"{target_unit}, giving us "
            f"{round_number(mme_drug[name] * total_per_day)} "
            f"mme/day. "
        )

        explanation += (
            f"Adding the mme/day of {name} to the total mme/day "
            f"gives us {round_number(mme_equivalent)} + "
            f"{round_number(mme_drug[name] * total_per_day)} = "
            f"{round_number(mme_equivalent + mme_drug[name] * total_per_day)} "
            f"mme/day.\n"
        )

        mme_equivalent += dose_per_day * mme_drug[name] * drug_mg

        mme_equivalent = round_number(mme_equivalent)

    explanation += f"The patient's mme/day is {mme_equivalent} mme/day."

    return {"Explanation": explanation, "Answer": mme_equivalent}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'Tapentadol Dose': (10, 'mg'),
            'Tapentadol Dose Per Day': (2, 'per day'),
            'HYDROcodone Dose': (30, 'mg'),
            'HYDROcodone Dose Per Day': (2, 'per day'),
            'FentANYL patch Dose': (70, 'mg'),
            'FentANYL patch Dose Per Day': (2, 'per day'),
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = mme_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_glasgow_coma_score_explanation
- translation

Date: March 2025
"""


def compute_glasgow_coma_score_explanation(input_variables):
    r"""
    Calculates the patient's Glasgow Coma Score and generates a detailed
    explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "insulin" (array): The patient's insulin level in the format (
            value, unit).
                - Value (float): The value of insulin level.
                - Unit (str): The unit of insulin level, eg. "µIU/mL",
                "pmol/L", and so on.
            - "glucose" (array): The patient's blood glucose level in the
            format (value, unit).
                - Value (float): The value of blood glucose level.
                - Unit (str): The unit of blood glucose level,
                eg. "mmol/L", "mEq/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Homeostatic Model Assessment
            for Insulin Resistance.

    Notes:
        - None

    Example:
        compute_glasgow_coma_score_explanation({'insulin': [756.0, 'pmol/L'],
        'glucose': [97.3, 'mg/dL']})

        output: "{'Explanation': "\n    The Glasgow Coma Scale (GCS) for
        assessing a patient's level of consciousness is shown below:\n
        \n       1. Best Eye Response: Spontaneously = +4 points, To verbal
        command = +3 points, To pain = +2 points, \n       No eye opening =
        +1 point\n       2. Best Verbal Response: Oriented = +5 points,
        Confused = +4 points, Inappropriate words = +3 points, \n
        Incomprehensible sounds = +2 points, No verbal response =
        +1 point\n       3. Best Motor Response: Obeys commands = +6 points,
        Localizes pain = +5 points, Withdrawal from pain = +4 points,
        \n Flexion to pain = +3 points, Extension to pain = +2 points,
        No motor response = +1 point\n\n For each criteria, if a patient's
        value is not mentioned/not testable in the note, \n we assume that
        it gets the full score for that attribute.  \n The total GCS score
        is calculated by summing the points for each of the three
        components.\n\n\n The current glasgow coma score is 0.\nBased
        on the patient note, the best eye response for the patient is 'eye
        opening to pain', and so we add 2 points making the current total 0
        + 2 = 2.\nBased on the patient note, the best verbal response for
        the patient is 'oriented', and so we add 5 points making
        the current total 2 + 5 = 7.\nBased on the patient note, the best
        motor response for the patient is 'localizes pain', and so we add 5
        points making the current total 7 + 5 = 12.\nHence, the patient's
        glasgow coma score is 12.\n", 'Answer': 12}"
    """

    glasgow_dictionary = {
        "best_eye_response": {
            "eyes open spontaneously": 4,
            "eye opening to verbal command": 3,
            "eye opening to pain": 2,
            "no eye opening": 1,
            "not testable": 4,
        },
        "best_verbal_response": {
            "oriented": 5,
            "confused": 4,
            "inappropriate words": 3,
            "incomprehensible sounds": 2,
            "no verbal response": 1,
            "not testable": 4,
        },
        "best_motor_response": {
            "obeys commands": 6,
            "localizes pain": 5,
            "withdrawal from pain": 4,
            "flexion to pain": 3,
            "extension to pain": 2,
            "no motor response": 1,
        },
    }

    best_eye_response_value = input_variables["best_eye_response"]
    best_verbal_response_value = input_variables["best_verbal_response"]
    best_motor_response_value = input_variables["best_motor_response"]

    eye_score = glasgow_dictionary["best_eye_response"][
        best_eye_response_value
    ]
    verbal_score = glasgow_dictionary["best_verbal_response"][
        best_verbal_response_value
    ]
    motor_score = glasgow_dictionary["best_motor_response"][
        best_motor_response_value
    ]

    glasgow_score = 0

    eye_point = "points" if eye_score == 0 or eye_score > 1 else "point"
    verbal_point = (
        "points" if verbal_score == 0 or verbal_score > 1 else "point"
    )
    motor_point = "points" if motor_score == 0 or motor_score > 1 else "point"

    explanation = """
    The Glasgow Coma Scale (GCS) for assessing a patient's level
    of consciousness is shown below:

       1. Best Eye Response: Spontaneously = +4 points, To verbal command =
        +3 points, To pain = +2 points, No eye opening = +1 point
       2. Best Verbal Response: Oriented = +5 points, Confused = +4 points,
        Inappropriate words = +3 points, Incomprehensible sounds = +2 points,
        No verbal response = +1 point
       3. Best Motor Response: Obeys commands = +6 points, Localizes pain = +5
        points, Withdrawal from pain = +4 points, Flexion to pain = +3 points,
        Extension to pain = +2 points, No motor response = +1 point

    For each criteria, if a patient's value is not mentioned/not testable in
    the note, we assume that it gets the full score for that attribute.
    The total GCS score is calculated by summing the points for each of
    the three components.\n\n
    """

    explanation += "The current glasgow coma score is 0.\n"

    if best_eye_response_value == 'not testable':
        explanation += (
            f"Based on the patient note, the best eye response for the "
            f"patient is '{best_eye_response_value}', and so we assume the "
            f"the patient can open his or her eyes spontaneously. "
            f"Hence, we add {eye_score} {eye_point}, making the current total "
            f"{glasgow_score} + {eye_score} = {glasgow_score + eye_score}.\n"
        )
        glasgow_score += eye_score
    else:
        explanation += (
            f"Based on the patient note, the best eye response for the "
            f"patient is '{best_eye_response_value}'"
            f", and so we add {eye_score} {eye_point} making the current "
            f"total {glasgow_score} + {eye_score} = "
            f"{glasgow_score + eye_score}.\n"
        )
        glasgow_score += eye_score

    if best_verbal_response_value == 'not testable':
        explanation += (
            f"Based on the patient note, the best verbal response for the "
            f"patient is '{best_verbal_response_value}', and so we assume "
            f"the the patient's verbal response is oriented."
            f" Hence, we add {verbal_score} {verbal_point}, making the "
            f"current total {glasgow_score} + {verbal_score} = "
            f"{glasgow_score + verbal_score}.\n"
        )
        glasgow_score += verbal_score
    else:
        explanation += (
            f"Based on the patient note, the best verbal response for the "
            f"patient is '{best_verbal_response_value}', and so we add "
            f"{verbal_score} {verbal_point} making the current total "
            f"{glasgow_score} + {verbal_score} = "
            f"{glasgow_score + verbal_score}.\n"
        )
        glasgow_score += verbal_score

    explanation += (
        f"Based on the patient note, the best motor response "
        f"for the patient is '{best_motor_response_value}', and so we add "
        f"{motor_score} {motor_point} making the current total "
        f"{glasgow_score} + {motor_score} = {glasgow_score + motor_score}.\n"
    )
    glasgow_score += motor_score
    explanation += (
        f"Hence, the patient's glasgow coma score is {glasgow_score}.\n"
    )

    return {"Explanation": explanation, "Answer": glasgow_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "best_eye_response": "eye opening to pain",
            "best_verbal_response": "oriented",
            "best_motor_response": "localizes pain",
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_glasgow_coma_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function framingham_calculator_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number


def framingham_calculator_explanation(input_variables):
    r"""
    Calculates the patient's corrected QT interval using the Framingham
    Formula and generates a detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "heart_rate" (tuple): The patient's heart rate in the format (
            value, unit).
                - Value (float): The value of the patient's heart rate.
                - Unit (str): The unit of heart rate should be "beats per
                minute".
            - "qt_interval" (tuple): The QT interval of 330 msec.
                - Value (float): The value of QT interval.
                - Unit (str): The unit of QT interval, "msec".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's corrected QT interval using
            the Bazett Formula.

    Notes:
        - None

    Example:
        framingham_calculator_explanation({"heart_rate": (81, "beats per
        minute"),"qt_interval": (330, "msec"),})

        output: "{'Explanation': "The corrected QT interval using the
        Framingham formula is computed as QTc = QT Interval + (154 * (1 -
        rr_interval_sec)), where QT interval is in msec, and RR interval is
        given as 60/( heart rate).\nThe patient's heart rate is
        81 beats per minute.\nThe QT interval is 330 msec.
        \nThe RR interval is computed as 60/(heart rate), and so the RR
        interval is 60/81 = 0.741. \nHence, plugging in these values,
        we will get 330/(154 * ( 1- 0.741 )) = 369.886. \nThe patient's
        corrected QT interval (QTc) is 369.886 msec.\n", 'Answer': 369.886}"
    """

    heart_rate = input_variables["heart_rate"][0]
    qt_interval = input_variables["qt_interval"][0]

    explanation = (
        "The corrected QT interval using the Framingham formula is computed "
        "as QTc = QT Interval + (154 * (1 - rr_interval_sec)), where QT "
        "interval is in msec, and RR interval is given as 60/(heart rate).\n"
    )

    explanation += (
        f"The patient's heart rate is {heart_rate} beats per minute.\n"
    )
    explanation += f"The QT interval is {qt_interval} msec.\n"

    rr_interval_sec = round_number(60 / heart_rate)
    explanation += (
        f"The RR interval is computed as 60/(heart rate), "
        f"and so the RR interval is 60/{heart_rate} = {rr_interval_sec}.\n"
    )

    qt_c = round_number(qt_interval + (154 * (1 - rr_interval_sec)))
    explanation += (
        f"Hence, plugging in these values, "
        f"we will get {qt_interval}/(154 * ( 1- {rr_interval_sec} )) = "
        f"{qt_c}.\n"
    )

    explanation += (
        f"The patient's corrected QT interval (QTc) is {qt_c} msec.\n"
    )

    return {"Explanation": explanation, "Answer": qt_c}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "heart_rate": (81, "beats per minute"),
            "qt_interval": (330, "msec"),
        },
        {
            "heart_rate": (96, "beats per minute"),
            "qt_interval": (330, "msec"),
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = framingham_calculator_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_homa_ir_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_homa_ir_explanation(input_variables):
    r"""
    Calculates the patient's Homeostatic Model Assessment for Insulin
    Resistance and generates a detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "insulin" (array): The patient's insulin level in the
            format (value, unit).
                - Value (float): The value of insulin level.
                - Unit (str): The unit of insulin level, eg. "µIU/mL",
                "pmol/L", and so on.
            - "glucose" (array): The patient's blood glucose level in the
            format (value, unit).
                - Value (float): The value of blood glucose level.
                - Unit (str): The unit of blood glucose level,
                eg. "mmol/L", "mEq/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Homeostatic Model Assessment
            for Insulin Resistance.

    Notes:
        - None

    Example:
        compute_homa_ir_explanation({'insulin': [756.0, 'pmol/L'],
        'glucose': [97.3, 'mg/dL']})

        output: "{'Explanation': "The formula for computing HOMA-IR score is
        (insulin (µIU/mL) * glucose mg/dL)/405. \nThe concentration of
        glucose is 97.3 mg/dL. \nPlugging into the formula will give us
        756.0 * 97.3/405 = 181.627.
        Hence, the patient's HOMA-IR score is 181.627. \n", 'Answer': 181.627}"
    """

    explanation = (
        "The formula for computing HOMA-IR score is (insulin ("
        "µIU/mL) * glucose mg/dL)/405.\n"
    )

    insulin = input_variables["insulin"][0]

    if input_variables["insulin"][1] == "µIU/mL":
        explanation += f"The concentration of insulin is {insulin} µIU/mL.\n"

    elif input_variables["insulin"][1] == "pmol/L":
        insulin = input_variables["insulin"][0] * 6
        explanation += (
            f"The concentration of insulin is {insulin} pmol/L. "
            f"We to need convert the concentration of insulin to pmol/L, "
            f"by multiplying by the conversion factor of 6.0 µIU/mL/pmol/L. "
            f"This makes the insulin concentration "
            f"{input_variables['insulin'][0]} * 6 µIU/mL/pmol/L = {insulin} "
            f"µIU/mL.\n"
        )

    elif input_variables["insulin"][1] == "ng/mL":
        insulin = input_variables["insulin"][0] * 24.8
        explanation += (
            f"The concentration of insulin is {insulin} ng/mL. "
            f"We to need convert the concentration of insulin to µIU/mL, "
            f"by multiplying by the conversion factor 24.8 µIU/mL/ng/mL. "
            f"This makes the insulin concentration "
            f"{input_variables['insulin'][0]} * 24.8 µIU/mL/ng/mL = "
            f"{insulin} ng/mL.\n"
        )

    glucose_exp, glucose = conversion_explanation(
        input_variables["glucose"][0],
        "glucose",
        180.16,
        None,
        input_variables["glucose"][1],
        "mg/dL",
    )

    explanation += glucose_exp + "\n"

    answer = round_number((insulin * glucose) / 405)

    explanation += (
        f"Plugging into the formula will give us "
        f"{insulin} * {glucose}/405 = {answer}. "
        f"Hence, the patient's HOMA-IR score is {answer}. \n"
    )

    return {"Explanation": explanation, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [{'insulin': [756.0, 'pmol/dL'], 'glucose': [97.3, 'mg/dL']}]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_homa_ir_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_cardiac_index_explanation
- translation

Date: March 2025
"""

from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_cardiac_index_explanation(input_variables):
    r"""
    Calculates the patient's cardiac index and generates a detailed
    explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "elevated_risk_surgery" (boolean): Elevated-risk surgery (
            intraperitoneal, intrathoracic, or suprainguinal vascular).
            - "ischemetic_heart_disease" (boolean): History of ischemic
            heart disease (history of myocardial infarction, positive
            exercise test, current chest pain due to myocardial ischemia,
            use of nitrate therapy, or ECG with pathological Q waves).
            - "congestive_heart_failure" (boolean): History of congestive
            heart failure (pulmonary edema, bilateral rales or S3 gallop,
            paroxysmal nocturnal dyspnea, or chest x-ray showing pulmonary
            vascular redistribution).
            - "cerebrovascular_disease" (boolean): History of
            cerebrovascular disease (prior transient ischemic
            attack or stroke).
            - "pre_operative_insulin_treatment" (boolean): Pre-operative
            treatment with insulin.
            - "pre_operative_creatinine" (tuple): The patient's
            pre-operative creatinine information in the format (value, unit).
                - Value (float): The value of pre-operative creatinine.
                - Unit (str): The unit of creatinine,
                which can be "mg/dL", "μmol/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's cardiac index.

    Notes:
        - None

    Example:
        compute_cardiac_index_explanation({
            "elevated_risk_surgery": False,
            "ischemetic_heart_disease": False,
            "congestive_heart_failure": False,
            "cerebrovascular_disease": False,
            "pre_operative_insulin_treatment": False,
            "pre_operative_creatinine": (1.0, "mg/dL"),
        })

        output: "{'Explanation': "\n    The criteria for the Revised Cardiac
        Risk Index (RCRI) are listed below:\n    \n       1. Elevated-risk
        surgery (intraperitoneal, intrathoracic, \n       or suprainguinal
        vascular): No = 0 points, Yes = +1 point\n       2. History of
        ischemic heart disease (history of myocardial \n       infarction,
        positive exercise test, current chest pain due to \n
        myocardial ischemia, use of nitrate therapy, or ECG with pathological
        \n       Q waves): No = 0 points, Yes = +1 point\n       3. History
        of congestive heart failure (pulmonary edema, bilateral \n
        rales or S3 gallop, paroxysmal nocturnal dyspnea, or chest x-ray
        \n       showing pulmonary vascular redistribution): \n       No = 0
        points, Yes = +1 point\n       4. History of cerebrovascular disease
        (prior transient ischemic \n       attack or stroke): No = 0 points,
        Yes = +1 point\n       5. Pre-operative treatment with insulin: No = 0
        points, Yes = +1 point\n       6. Pre-operative creatinine >2 mg/dL
        (176.8 μmol/L): No = 0 points, \n       Yes = +1 point\n    \n    The
        total score is calculated by summing the points for each criterion.\n
        \n\n    The current cardiac risk index is 0.\nThe patient note reports
        elevated risk surgery as 'absent' for the patient. This means that the
        total score remains unchanged at 0.\nThe patient note reports
        ischemetic heart disease as 'absent' for the patient. This means that
        the total score remains unchanged at 0.\nThe patient note reports
        congestive heart failure as 'absent' for the patient. This means that
        the total score remains unchanged at 0.\nThe patient note reports
        cerebrovascular disease as 'absent' for the patient. This means that
        the total score remains unchanged at 0.\nThe patient note reports
        pre-operative insulin treatment as 'absent' for the patient. This
        means that the total score remains unchanged at 0.\nThe concentration
        of Pre-Operative Creatinine is 1.0 mg/dL. The patient has pre-operative
         creatinine <= 2 mg/dL, so we keep the score the same at 0.\n\nThe
         cardiac risk index score is 0.\n", 'Answer': 0}"
    """

    # List of parameters and their default values
    parameters = {
        'elevated_risk_surgery': "elevated risk surgery",
        'ischemetic_heart_disease': "ischemetic heart disease",
        'congestive_heart_failure': "congestive heart failure",
        'cerebrovascular_disease': "cerebrovascular disease",
        'pre_operative_insulin_treatment': "pre-operative insulin treatment",
        'pre_operative_creatinine': "pre-operative creatinine",
    }

    output = """
    The criteria for the Revised Cardiac Risk Index (RCRI) are listed below:
    
       1. Elevated-risk surgery (intraperitoneal, intrathoracic, 
       or suprainguinal vascular): No = 0 points, Yes = +1 point
       2. History of ischemic heart disease (history of myocardial 
       infarction, positive exercise test, current chest pain due to 
       myocardial ischemia, use of nitrate therapy, or ECG with pathological 
       Q waves): No = 0 points, Yes = +1 point
       3. History of congestive heart failure (pulmonary edema, bilateral 
       rales or S3 gallop, paroxysmal nocturnal dyspnea, or chest x-ray 
       showing pulmonary vascular redistribution): 
       No = 0 points, Yes = +1 point
       4. History of cerebrovascular disease (prior transient ischemic 
       attack or stroke): No = 0 points, Yes = +1 point
       5. Pre-operative treatment with insulin: No = 0 points, Yes = +1 point
       6. Pre-operative creatinine >2 mg/dL (176.8 μmol/L): No = 0 points, 
       Yes = +1 point
    
    The total score is calculated by summing the points for each criterion.\n\n
    """

    # Initializing scores and output explanation
    cri = 0
    output += "The current cardiac risk index is 0.\n"

    for param_name, full_name in parameters.items():
        param_value = input_variables.get(param_name)

        # If parameter is missing, assume it as False
        if param_value is None:
            output += (
                f"The patient note does not mention about {full_name} "
                f"and is assumed to be absent. "
            )
            input_variables[param_name] = False
            param_value = False
        elif param_name != 'pre_operative_creatinine':
            value = 'absent' if not param_value else 'present'
            output += (
                f"The patient note reports {full_name} as '{value}' "
                f"for the patient. "
            )
        elif param_name == 'pre_operative_creatinine':
            explanation, param_value = conversion_explanation(
                param_value[0],
                "Pre-Operative Creatinine",
                113.12,
                None,
                param_value[1],
                "mg/dL",
            )
            input_variables['pre_operative_creatinine'] = [
                param_value,
                "mg/dL",
            ]
            output += explanation

        if param_name == 'pre_operative_creatinine':
            if param_value > 2:
                output += (
                    f"The patient has pre-operative creatinine > 2 "
                    f"mg/dL, so we increment the score by one and the "
                    f"current total will be {cri} + 1 = {cri + 1}.\n"
                )
                cri += 1
            else:
                output += (
                    f"The patient has pre-operative creatinine <= 2 "
                    f"mg/dL, so we keep the score the same at {cri}.\n"
                )
            continue

        if param_value:
            output += (
                f"This means that we increment the score by one and "
                f"the current total will be {cri} + 1 = {cri + 1}.\n"
            )
            cri += 1
        else:
            output += (
                f"This means that the total score "
                f"remains unchanged at {cri}.\n"
            )

    output += f"\nThe cardiac risk index score is {cri}.\n"

    return {"Explanation": output, "Answer": cri}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "elevated_risk_surgery": False,
            "ischemetic_heart_disease": False,
            "congestive_heart_failure": False,
            "cerebrovascular_disease": False,
            "pre_operative_insulin_treatment": False,
            "pre_operative_creatinine": (1.0, "mg/dL"),
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_cardiac_index_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function glasgow_bleeding_score_explanation
- translation

Date: March 2025
"""

from .utils.unit_converter_new import (
    conversion_explanation,
)


def glasgow_bleeding_score_explanation(input_parameters):
    explanation = r"""
    The Glasgow-Blatchford Score (GBS) for assessing the severity of 
    gastrointestinal bleeding is shown below:

       1. Hemoglobin level (g/dL): Enter value (norm: 12-17 g/dL)
       2. BUN level (mg/dL): Enter value (norm: 8-20 mg/dL)
       3. Initial systolic blood pressure (mm Hg): Enter value (norm: 
       100-120 mm Hg)
       4. Sex: Female = +1 point, Male = 0 points
       5. Heart rate ≥100: No = 0 points, Yes = +1 point
       6. Melena present: No = 0 points, Yes = +1 point
       7. Recent syncope: No = 0 points, Yes = +2 points
       8. Hepatic disease history: No = 0 points, Yes = +2 points
       9. Cardiac failure present: No = 0 points, Yes = +2 points
    
    The total score is calculated by summing the points for each criterion (
    additional lab values may also be factored into the score).\n\n
    """

    score = 0

    hemoglobin_exp, hemoglobin = conversion_explanation(
        input_parameters["hemoglobin"][0],
        "hemoglobin",
        64500,
        None,
        input_parameters["hemoglobin"][1],
        "g/dL",
    )
    bun_exp, bun = conversion_explanation(
        input_parameters["bun"][0],
        "BUN",
        28.08,
        None,
        input_parameters["bun"][1],
        "mg/dL",
    )
    gender = input_parameters["sex"]
    systiolic_bp = input_parameters["sys_bp"][0]
    heart_rate = input_parameters["heart_rate"][0]

    explanation += (
        f"The current glasgow bleeding score is 0. The patient's "
        f"gender is {gender}.\n"
    )
    explanation += hemoglobin_exp

    if gender == "Male":
        if 12 < hemoglobin <= 13:
            explanation += (
                f"Because the patient is a male and the "
                f"hemoglobin concentration is between 12 and 13 "
                f"g/dL, we add one point, making the current "
                f"score {score} + 1 = {score + 1}.\n"
            )
            score += 1
        elif 10 <= hemoglobin < 12:
            explanation += (
                f"Because the patient is a male and the "
                f"hemoglobin concentration is between 10 and 12 "
                f"g/dL, we add three points, making the current "
                f"score {score} + 3 = {score + 3}.\n"
            )
            score += 3
        elif hemoglobin < 10:
            explanation += (
                f"Because the patient is a male and the "
                f"hemoglobin concentration is less than "
                f"10 and 12 g/dL, we add six points, "
                f"making the current score "
                f"{score} + 6 = {score + 6}.\n"
            )
            score += 6
        elif hemoglobin > 13:
            explanation += (
                f"Because the patient is a male and the "
                f"hemoglobin concentration is greater than 13 "
                f"g/dL, we do not add any points, "
                f"keeping the current score at {score}.\n"
            )

    else:
        if 10 < hemoglobin <= 12:
            explanation += (
                f"Because the patient is a female "
                f"and the hemoglobin concentration is between 10 "
                f"and 12 mg/dL, we add one point, making the "
                f"current score {score} + 1 = {score + 1}.\n"
            )
            score += 1
        elif hemoglobin < 10:
            explanation += (
                f"Because the patient is a female and the "
                f"hemoglobin concentration is less "
                f"than 10 mg/dL, we add three points, "
                f"making the current score "
                f"{score} + 3 = {score + 3}.\n"
            )
            score += 6
        elif hemoglobin > 12:
            explanation += (
                f"Because the patient is a female and the "
                f"hemoglobin concentration is greater than 12 "
                f"mg/dL, we do not add any points, keeping the "
                f"current score at {score}.\n"
            )

    explanation += bun_exp

    if 18.2 <= bun < 22.4:
        explanation += (
            f"The BUN concentration is between 18.2 and 22.4 "
            f"mg/dL, and so we add two points, "
            f"making the current score "
            f"{score} + 2 = {score + 2}.\n"
        )
        score += 2
    elif 22.4 <= bun < 28:
        explanation += (
            f"The BUN concentration is between "
            f"22.4 and 28 mg/dL, and so we add three points, "
            f"making the current score "
            f"{score} + 3 = {score + 3}.\n"
        )
        score += 3
    elif 28 <= bun < 70:
        explanation += (
            f"The BUN concentration is between 28 and 70 mg/dL, "
            f"and so we add four points, making the current score"
            f" {score} + 4 = {score + 4}.\n"
        )
        score += 4
    elif bun > 70:
        explanation += (
            f"The BUN concentration is greater than 70 mg/dL, "
            f"and so we add six points, making the current score"
            f" {score} + 6 = {score + 6}.\n"
        )
        score += 6
    elif bun < 18.2:
        explanation += (
            f"The BUN concentration is less than 18.2 mg/dL, "
            f"and so we do not make any changes to the score, "
            f"keeping the score at {score}.\n"
        )

    explanation += f"The patient's blood pressure is {systiolic_bp} mm Hg. "

    if 100 <= systiolic_bp < 110:
        explanation += (
            f"Because the patient's systolic blood pressure is "
            f"between 100 and 110 mm Hg, we increase the "
            f"score by one point, making the current score "
            f"{score} + 1 = {score + 1}.\n"
        )
        score += 1
    elif 90 <= systiolic_bp < 100:
        explanation += (
            f"Because the patient's systolic blood pressure is "
            f"between 90 and 100 mm Hg, we increase the score by "
            f"two points, making the current score "
            f"{score} + 2 = {score + 2}.\n"
        )
        score += 2
    elif systiolic_bp < 90:
        explanation += (
            f"Because the patient's systolic blood pressure is "
            f"less than 90 mm Hg, we increase the score by three "
            f"points, making the current score "
            f"{score} + 3 = {score + 3}.\n"
        )
        score += 3
    elif systiolic_bp >= 110:
        explanation += (
            f"Because the patient's systolic blood pressure is "
            f"greater than or equal to 110 mm Hg, we do not add "
            f"points to the score, keeping the current score at"
            f" {score} + 3 = {score + 3}.\n"
        )

    explanation += (
        f"The patient's heart rate is {heart_rate} beats per " f"minute. "
    )

    if heart_rate >= 100:
        explanation += (
            f"Because the heart rate is greater or equal to than "
            f"100 beats per minute, we increase the score by one "
            f"point, making the current score {score} + 1 ="
            f" {score + 1}.\n"
        )
        score += 1
    else:
        explanation += (
            f"Because the heart rate is less than 100 beats per "
            f"minute, we do not change the score, keeping the "
            f"current score at {score}.\n"
        )

    default_parameters = {
        "melena_present": "melena",
        "syncope": "recent syncope",
        "hepatic_disease_history": "hepatic disease history",
        "cardiac_failure": "cardiac failure",
    }

    for parameter in default_parameters:
        if parameter not in input_parameters:
            explanation += (
                f"The patient's status for"
                f" {default_parameters[parameter]} is missing "
                f"from the patient note and so we assume it is "
                f"absent from the patient.\n"
            )
            input_parameters[parameter] = False
            explanation += (
                f"Hence, we do not add any points to the score, "
                f"keeping it at {score}.\n"
            )

        elif (
            parameter
            in ['syncope', 'hepatic_disease_history', 'cardiac_failure']
            and input_parameters[parameter]
        ):
            explanation += (
                f"The patient has a"
                f" {default_parameters[parameter]}, and so we "
                f"add two points to the current total, "
                f"making the current total "
                f"{score} + 2 =  {score + 2}.\n"
            )
            score += 2

        elif input_parameters[parameter]:
            explanation += (
                f"The patient has "
                f"{default_parameters[parameter]} and so we add "
                f"one point to the current total, making the "
                f"current total {score} + 1 =  {score + 1}.\n"
            )
            score += 1

        else:
            explanation += (
                f"The patient's status for "
                f"{default_parameters[parameter]} is reported to "
                f"be absent for the patient, and "
                f"so we do not add any points, "
                f"keeping the current total at {score}.\n"
            )

    explanation += f"The patient's Glasgow Bleeding Score is {score}.\n"

    return {"Explanation": explanation, "Answer": score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "hemoglobin": (12, 'g/dL'),
            "bun": (10, 'mg/dL'),
            "sys_bp": (70, "mm"),
            "sex": "Male",
            "heart_rate": (92.0, "breaths per minute"),
            "platelet_count": (277000.0, 'µL'),
            "melena_present": False,
            "syncope": False,
            "hepatic_disease_history": False,
            "cardiac_failure": False,
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = glasgow_bleeding_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function targetweight_explanation
- translation

Date: March 2025
"""

from .utils.height_conversion import (
    convert_height_to_target_unit_explanation,
)
from .utils.rounding import round_number


def targetweight_explanation(input_variables):
    r"""
    Calculates the patient's delta gap and generates a detailed explanatory
    text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "body_mass_index" (tuple): The patient's BMI in the
            format (value, unit).
                - Value (float): The patient's BMI value.
                - Unit (str): The unit of BMI.
            - "height" (tuple): The patient's height information in the
            format (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height, which can be "cm"
                (centimeters) or "in" (inches).

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's target weight.

    Notes:
        - Uses the `height_conversion_explanation` function to convert
        height to meters.

    Example:
        targetweight_explanation({"body_mass_index": (20.1, "kg/m^2"),
        "height": (72, "in"),})

        output: "{'Explanation': "The patient's target bmi is 20.1 kg/m^2.
        The patient's height is 72 in, which is 72 in * 0.0254 m / in =
        1.829 m. From this, the patient's target weight is 20.1 kg/m^2 *
        1.829 m * 1.829 m = 67.239 kg. ", 'Answer': 67.239}"
    """

    bmi = input_variables["body_mass_index"][0]
    height_exp, height = convert_height_to_target_unit_explanation(
        input_variables["height"]
    )
    target_weight_val = round_number(bmi * (height * height))

    explanation = ""

    explanation += f"The patient's target bmi is {bmi} kg/m^2. "

    explanation += f"{height_exp}"

    explanation += (
        f"From this, the patient's target weight is {bmi} kg/m^2 "
        f"* {height} m * {height} m = {target_weight_val} kg. "
    )

    return {"Explanation": explanation, "Answer": target_weight_val}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "body_mass_index": (20.1, "kg/m^2"),
            "height": (72, "in"),
        },
        {
            "body_mass_index": (22.0, "kg/m^2"),
            "height": (75, "in"),
        },
    ]
    # {'Body Mass Index (BMI)': [20.1, 'kg/m^2'], 'height': [72, 'in']}
    # {'Body Mass Index (BMI)': [22.0, 'kg/m^2'], 'height': [75, 'in']}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = targetweight_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function weight_conversion_explanation
- translation

Date: March 2025
"""

from .rounding import round_number


def weight_conversion_explanation(weight_info):
    r"""
    Converts a patient's weight from different units (pounds, grams,
    or kilograms) to kilograms
    and generates an explanatory text.

    Parameters:
        weight_info (tuple): A tuple containing two elements:
            - First element (float): The patient's weight value.
            - Second element (str): The unit of weight, which can be one of
            the following:
                - "lbs" for pounds.
                - "g" for grams.
                - "kg" for kilograms.

    Returns:
        tuple: Contains two elements:
            - First element (str): An explanatory text describing the
            conversion process and result.
            - Second element (float): The converted weight value (in
            kilograms).

    Notes:
        - Uses the `round_number` function to round the result.
        - If the input unit is not "lbs," "g," or "kg," the function
        defaults to returning the input value as kilograms.

    Example:
        convert_weight((150, "lbs"))
        output: "The patient's weight is 150 lbs so this converts to
        150 lbs * 0.453592 kg/lbs = 68.04 kg."
    """
    assert len(weight_info) == 2
    weight = weight_info[0]
    weight_label = weight_info[1]

    answer = round_number(weight * 0.453592)

    if weight_label == "lbs":
        return (
            f"The patient's weight is {weight} lbs so this converts to "
            f"{weight} lbs * 0.453592 kg/lbs = {answer} kg. ",
            answer,
        )
    elif weight_label == "g":
        return (
            f"The patient's weight is {weight} g so this converts to "
            f"{weight} lbs * kg/1000 g = "
            f"{round_number(weight / 1000)} kg. ",
            weight / 1000,
        )
    else:
        return f"The patient's weight is {weight} kg. ", weight



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_has_bled_score_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)


def compute_has_bled_score_explanation(input_variables):
    r"""
    Calculates the patient's Centor Score and generates a detailed
    explanatory text.

        {
            "age": (45, 'years'),
            "hypertension": False,
            "liver_disease_has_bled": False,
            "renal_disease_has_bled": False,
            "stroke": False,
            "prior_bleeding": False,
            "labile_inr": False,
            "medications_for_bleeding": False,
            "alcoholic_drinks": 6
        }

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "age" (tuple): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".
            - "hypertension" (boolean): Hypertension (Uncontrolled,
            >160 mmHg systolic).
            - "liver_disease_has_bled" (boolean): Liver disease
            (Cirrhosis or bilirubin > 2x normal with AST/ALT/AP > 3x normal)
            - "renal_disease_has_bled" (boolean): Renal disease (Dialysis,
            transplant, Cr >2.26 mg/dL or > 200 µmol/L).
            - "stroke" (boolean): Stroke history.
            - "prior_bleeding" (boolean): Prior major bleeding or
            predisposition to bleeding.
            - "labile_inr" (boolean): Labile INR (Unstable/high INRs,
            time in therapeutic range < 60%).
            - "medications_for_bleeding" (boolean): Medication usage
            predisposing to bleeding (Aspirin, clopidogrel, NSAIDs).
            - "alcoholic_drinks" (int): Alcohol use (≥8 drinks/week).

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's HAS-BLED Score.

    Notes:
        - None

    Example:
        compute_has_bled_score_explanation()

        output: ""
    """

    explanation = """
    The criteria for the HAS-BLED score are listed below below:
    
       1. Hypertension (Uncontrolled, >160 mmHg systolic): 
       No = 0 points, Yes = +1 point
       2. Renal disease (Dialysis, transplant, Cr >2.26 mg/dL or > 200 
       µmol/L): No = 0 points, Yes = +1 point
       3. Liver disease (Cirrhosis or bilirubin > 2x normal with AST/ALT/AP 
       > 3x normal): No = 0 points, Yes = +1 point
       4. Stroke history: No = 0 points, Yes = +1 point
       5. Prior major bleeding or predisposition to bleeding: 
       No = 0 points, Yes = +1 point
       6. Labile INR (Unstable/high INRs, time in therapeutic range < 60%): 
       No = 0 points, Yes = +1 point
       7. Age >65: No = 0 points, Yes = +1 point
       8. Medication usage predisposing to bleeding (Aspirin, clopidogrel, 
       NSAIDs): No = 0 points, Yes = +1 point
       9. Alcohol use (≥8 drinks/week): No = 0 points, Yes = +1 point
    
    The total HAS-BLED score is calculated by summing the points for each 
    criterion.
    """

    has_bled_score = 0

    num_alcolic_drinks = input_variables["alcoholic_drinks"]

    explanation += "The current HAS-BLED score is 0.\n"
    age_explanation, age_value = age_conversion_explanation(
        input_variables["age"]
    )
    explanation += age_explanation

    if age_value > 65:
        explanation += (
            f"Because the patient's age is greater than 65 years, "
            f"we increment the score by 1, making the "
            f"current score {has_bled_score} + 1 = "
            f"{has_bled_score + 1}.\n"
        )
        has_bled_score += 1
    else:
        explanation += (
            f"Because the patient's age is less than "
            f"66 years, we don't change the score, "
            f"keeping the current score at {has_bled_score}.\n"
        )

    if num_alcolic_drinks >= 8:
        explanation += (
            f"The patient has {num_alcolic_drinks} drinks a "
            f"week. Because the patient has at least 8 alcoholic "
            f"drinks a week, we increment the score by 1, "
            f"making the current score {has_bled_score} + 1 "
            f"= {has_bled_score + 1}.\n"
        )
        has_bled_score += 1
    else:
        explanation += (
            f"The patient has {num_alcolic_drinks} drinks a "
            f"week. Because the patient has less than 8 "
            f"alcoholic drinks a week, we don't change the "
            f"score, keeping the current score "
            f"at {has_bled_score}.\n"
        )

    default_parameters_set = {
        "hypertension": "hypertension",
        "liver_disease_has_bled": "liver disease",
        "renal_disease_has_bled": "renal disease",
        "stroke": "stroke history",
        "prior_bleeding": "prior bleeding",
        "labile_inr": "labile inr",
        "medications_for_bleeding": "medications " "for bleeding",
    }

    for parameter, name in default_parameters_set.items():
        if parameter not in input_variables:
            explanation += (
                f"The issue, {name}, is missing from "
                f"the patient note and so we assume it to "
                f"be absent and so we do not change the score, "
                f"keeping the current score at {has_bled_score}.\n"
            )
            input_variables[parameter] = False
        elif not input_variables[parameter]:
            explanation += (
                f"The issue, {name}, is reported to be absent "
                f"for the patient and so we do not change "
                f"the score, keeping the current score "
                f"at {has_bled_score}.\n"
            )
        else:
            explanation += (
                f"The issue, {name}, is reported to be present "
                f"for the patient note and so we increase the "
                f"score by 1, making the current score "
                f"{has_bled_score} + 1 = {has_bled_score + 1}.\n"
            )
            has_bled_score += 1

    explanation += (
        f"Hence, the patient's HAS-BLED score " f"is {has_bled_score}.\n"
    )

    return {"Explanation": explanation, "Answer": has_bled_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "age": (45, 'years'),
            "hypertension": False,
            "liver_disease_has_bled": False,
            "renal_disease_has_bled": False,
            "stroke": False,
            "prior_bleeding": False,
            "labile_inr": False,
            "medications_for_bleeding": False,
            "alcoholic_drinks": 6,
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_has_bled_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_albumin_delta_ratio_explanation
- translation

Date: March 2025
"""

from .albumin_corrected_delta_gap import (
    compute_albumin_corrected_delta_gap_explanation,
)
from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_albumin_delta_ratio_explanation(input_parameters):
    r"""
    Calculates the patient's albumin delta ratio and generates a detailed
    explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "sodium" (array): The patient's blood sodium level in the
            format (value, unit).
                - Value (float): The blood sodium level.
                - Unit (str): The unit of blood sodium level.
            - "chloride" (array): The patient's chloride level in
            the format (value, unit).
                - Value (float): The value of chloride level.
                - Unit (str): The unit of chloride level, eg. "mmol/L",
                "mEq/L", and so on.
            - "bicarbonate" (array): The patient's bicarbonate level in
            the format (value, unit).
                - Value (float): The value of bicarbonate level.
                - Unit (str): The unit of bicarbonate level, eg. "mmol/L",
                "mEq/L", and so on.
            - "albumin" (array): The patient's albumin concentration in
            the format (value, unit).
                - Value (float): The numerical albumin concentration value.
                - Unit (str): The unit of albumin concentration, eg. "g/L",
                "mg/dL", "g/mL" and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's albumin delta ratio.

    Notes:
        - None

    Example:
        compute_albumin_delta_ratio_explanation({'chloride': (105.0,
        'mEq/L'), 'bicarbonate': (28.0, 'mEq/L'), 'albumin': (3.9, 'g/dL'),
        'sodium': (140.0, 'mEq/L')})

        output: "{'Explanation': "The formula for computing the albumin
        corrected delta ratio is albumin corrected delta gap (mEq/L)/(24 -
        bicarbonate mEq/L).\nTo compute the formula of albumin corrected
        delta gap, the formula is albumin corrected anion gap (in mEq/L) -
        12.\nThe formula for computing a patient's albumin corrected anion
        gap is: anion_gap (in mEq/L) + 2.5 * (4 - albumin (in g/dL)).\nThe
        formula for computing a patient's anion gap is: sodium (mEq/L) - (
        chloride (mEq/L)+ bicarbonate (mEq/L)).\nThe concentration of
        sodium is 140.0 mEq/L. \nThe concentration of chloride is 105.0
        mEq/L. \nThe concentration of bicarbonate is 28.0 mEq/L. \nPlugging
        in these values into the anion gap formula gives us 140.0 mEq/L - (
        105.0 mEq/L + 28.0 mEq/L) = 7.0 mEq/L. Hence, The patient's anion
        gap is 7.0 mEq/L.\nThe concentration of albumin is 3.9 g/dL.
        Plugging in these values into the albumin corrected anion gap
        formula, we get 7.0 (mEq/L) + 2.5 * (4 - 3.9 ( in g/dL)) = 7.25
        mEq/L. Hence, the patient's albumin corrected anion gap is 7.25
        mEq/L.\nPlugging in 7.25 mEq/L for the anion gap into the albumin
        corrected delta gap formula, we get 7.25 - 12 = -4.75 mEq/L. Hence,
        the patient's albumin corrected delta gap is -4.75 mEq/L.\nPlugging
        in the albumin corrected delta gap and the bicarbonate
        concentration into the albumin corrected delta ratio formula,
        we get -4.75 mEq/L / -4.0 mEq/L = 1.188. The patient's albumin
        corrected delta ratio is 1.188.\n", 'Answer': 1.188}"
    """

    albumin_delta_gap_resp = compute_albumin_corrected_delta_gap_explanation(
        input_parameters
    )

    bicarbonate_exp, bicarbonate_val = conversion_explanation(
        input_parameters["bicarbonate"][0],
        "bicarbonate",
        61.02,
        1,
        input_parameters["bicarbonate"][1],
        "mEq/L",
    )

    explanation = (
        "The formula for computing the albumin corrected delta ratio is "
        "albumin corrected delta gap (mEq/L)/(24 - bicarbonate mEq/L).\n"
    )

    albmin_corrected_delta_gap_val = albumin_delta_gap_resp["Answer"]

    explanation += f"{albumin_delta_gap_resp['Explanation']}"

    final_answer = round_number(
        albumin_delta_gap_resp['Answer'] / (24 - bicarbonate_val)
    )

    explanation += (
        f"Plugging in the albumin corrected delta gap and the bicarbonate "
        f"concentration into the albumin "
        f"corrected delta ratio formula, we get "
        f"{albmin_corrected_delta_gap_val} mEq/L / "
        f"{24 - bicarbonate_val} mEq/L = {final_answer}. "
    )
    explanation += (
        f"The patient's albumin corrected delta ratio is {final_answer}.\n"
    )

    return {"Explanation": explanation, "Answer": final_answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "chloride": (100.0, "mmol/L"),
            "bicarbonate": (19.0, "mmol/L"),
            "albumin": (4.4, "g/dL"),
            "sodium": (134.0, "mmol/L"),
        },
        {
            "chloride": (105.0, "mEq/L"),
            "bicarbonate": (28.0, "mEq/L"),
            "albumin": (3.9, "g/dL"),
            "sodium": (140.0, "mEq/L"),
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_albumin_delta_ratio_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_heart_score_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)


def compute_heart_score_explanation(input_parameters):
    r"""
    Calculates the patient's heart score and generates a detailed
    explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "age" (array): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".
            - "hypertension" (boolean): The patient's history of hypertension.
            - "history" (str): 'Slightly suspicious', 'Moderately
            suspicious', or 'Highly suspicious'.
            - "diabetes_mellitus" (boolean): The patient's diabetes status.
            - "smoking" (boolean): The patient's history of smoking.
            - "family_with_cvd" (boolean): The patient's parent or sibling
            with Cardiovascular disease before age 65.
            - "atherosclerotic_disease" (boolean): The patient's
            atherosclerotic disease.
            - "initial_troponin" (str): The patient's initial troponin:
            "less than or equal to normal limit",
                "between the normal limit or up to three times the normal
                limit",
                or "greater than three times normal limit".
            - "electrocardiogram" (str): The patient's electrocardiogram
            status: "Normal", "Non-specific repolarization disturbance",
            or "Significant ST deviation".
            - "hypercholesterolemia" (boolean): The patient's
            hypercholesterolemia status.
            - "obesity" (boolean): The patient's obesity (BMI > 30 kg/m²).
            - "risk_factors" (dict): Any above risk factors == True should
            be added in this dictionary.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's corrected calcium
            concentration (in mg/dL).

    Notes:
        - Uses the `conversion_explanation` function to convert Albumin
        level to standard unit g/dL.
        - Uses the `conversion_explanation` function to convert Calcium
        level to standard unit mg/dL.

    Example:
        test_case = {'age': [60, 'years'],
            'hypertension': False,
            'history': 'Slightly suspicious',
            'diabetes_mellitus': False,
            'smoking': False,
            'family_with_cvd': True,
            'atherosclerotic_disease': False,
            'initial_troponin': 'less than or equal to normal limit',
            'electrocardiogram': 'Normal',
            'hypercholesterolemia': False,
            'obesity': False,
            'risk_factors': {'family_with_cvd'}
         }
        compute_heart_score_explanation(test_case)

        output: "{'Explanation': "\nThe HEART Score for risk stratification
        in patients with chest pain is shown below: \n    \n       1.
        History: Slightly suspicious = 0 points, Moderately suspicious = +1
        point, Highly suspicious = +2 points\n 2. EKG: Normal = 0 points,
        Non-specific repolarization disturbance = +1 point, Significant ST
        deviation = +2 points\n 3. Age: <45 years = 0 points, 45-64 years =
        +1 point, ≥65 years = +2 points\n 4. Risk factors (HTN,
        hypercholesterolemia, DM, obesity (BMI >30 kg/m²), smoking (current
        or cessation within 3 months), positive family history of
        cardiovascular disease before age 65, atherosclerotic disease such
        as prior MI, PCI/CABG, CVA/TIA, or peripheral arterial disease): No
        known risk factors = 0 points, 1-2 risk factors = +1 point, ≥3 risk
        factors or history of atherosclerotic disease = +2 points\n 5.
        Initial troponin level: ≤normal limit = 0 points, 1-3x normal limit
        = +1 point, >3x normal limit = +2 points \n    \n       The total
        score is calculated by summing the points for each criterion.\n\n\n
        The current HEART Score is 0.\nThe value of 'history' in the
        patient's note is determined to be 'Slightly suspicious'. Based on
        the HEART Score criteria, 0 points are added for 'history', keeping
        the current total at 0.\nThe value of 'electrocardiogram' in the
        patient's note is determined to be 'Normal'. Based on the HEART
        Score criteria, 0 points are added for 'electrocardiogram', keeping
        the current total at 0.\nThe patient is 60 years old. The patient's
        age is between 45 and 65 years and so we increment the current
        total by 1, making the current total 0 + 1 = 1.\nThe following risk
        factor(s) are present based on the patient's note: family with cvd.
        The following risk factor(s) are mentioned in the patient's note,
        but these risk factors are noted to be absent from the patient:
        hypertension, hypercholesterolemia, diabetes mellitus, obesity,
        smoking, atherosclerotic disease. Based on the HEART Score risk
        factors criteria, 1 risk factors are present and so 1 point is
        added for the risk factors criteria, making the current total,
        1 + 1 = 2.\nThe value of 'initial troponin' in the patient's
        note is determined to be 'less than or equal to normal limit'.
        Based on the HEART Score criteria, 0 points are added for 'initial
        troponin', keeping the current total at 2.\nBased on the patient's
        data, the HEART Score is 2.\n", 'Answer': 2}"
    """

    explanation = """
       The HEART Score for risk stratification in patients with
       chest pain is shown below:

       1. History: Slightly suspicious = 0 points, Moderately suspicious =
        +1 point, Highly suspicious = +2 points
       2. EKG: Normal = 0 points, Non-specific repolarization disturbance =
        +1 point, Significant ST deviation = +2 points
       3. Age: <45 years = 0 points, 45-64 years = +1 point, ≥65 years =
        +2 points
       4. Risk factors (HTN, hypercholesterolemia, DM, obesity (BMI >30 kg/m²),
        smoking (current or cessation within 3 months), positive family
        history of cardiovascular disease before age 65, atherosclerotic
        disease such as prior MI, PCI/CABG, CVA/TIA, or peripheral arterial
        disease): No known risk factors = 0 points, 1-2 risk factors = +1
        point, ≥3 risk factors or history of atherosclerotic disease = +2
        points
       5. Initial troponin level: ≤normal limit = 0 points, 1~3x normal limit
        = +1 point, >3x normal limit = +2 points

       The total score is calculated by summing the points for each criterion.
    """

    # Define parameters and their default values
    parameters = {
        'history': {
            'Slightly suspicious': 0,
            'Moderately suspicious': 1,
            'Highly suspicious': 2,
        },
        'electrocardiogram': {
            'Normal': 0,
            'Non-specific repolarization disturbance': 1,
            'Significant ST deviation': 2,
        },
        'age': {'< 45': 0, '45 - 65': 1, '> 65': 2},
        'risk_factors': {
            'hypertension': 1,
            'hypercholesterolemia': 1,
            'diabetes_mellitus': 1,
            'obesity': 1,
            'smoking': 1,
            'family_with_cvd': 1,
            'atherosclerotic_disease': 1,
        },
        'initial_troponin': {
            'less than or equal to normal limit': 0,
            'between the normal limit or up to '
            'three times the normal limit': 1,
            'greater than three times normal limit': 2,
        },
    }

    factor_name = {
        'hypertension': 'hypertension',
        'hypercholesterolemia': 'hypercholesterolemia',
        'diabetes_mellitus': 'diabetes mellitus',
        'obesity': 'obesity',
        'smoking': 'smoking',
        'family_with_cvd': "family with cvd",
        'atherosclerotic_disease': "atherosclerotic disease",
        'initial_tropopin': 'initial tropopin',
    }

    default_value = {
        'history': 'Slightly suspicious',
        'electrocardiogram': 'Normal',
        'initial_troponin': 'less than or equal to normal limit',
    }

    # Initialize total score and output explanation
    total_score = 0
    explanation += "The current HEART Score is 0.\n"

    for param, options in parameters.items():
        param_value = input_parameters.get(param)

        if param == 'risk_factors':
            present_factors = [
                factor
                for factor in options.keys()
                if input_parameters.get(factor)
            ]
            present_factors_names = [
                factor_name[factor]
                for factor in options.keys()
                if input_parameters.get(factor)
            ]

            if present_factors:
                explanation += (
                    f"The following risk factor(s) are present based on the "
                    f"patient's note: "
                    f"{', '.join(present_factors_names)}. "
                )

            present_but_false = [
                factor
                for factor in options.keys()
                if factor in input_parameters and not input_parameters[factor]
            ]
            present_but_false_names = [
                factor_name[factor]
                for factor in options.keys()
                if factor in input_parameters and not input_parameters[factor]
            ]

            if present_but_false:
                explanation += (
                    f"The following risk factor(s) are mentioned in the "
                    f"patient's note, but these risk factors are noted "
                    f"to be absent from the patient: "
                    f"{', '.join(present_but_false_names)}. "
                )
                for item in present_but_false:
                    input_parameters[item] = False

            missing_factors = [
                factor
                for factor in options.keys()
                if factor not in input_parameters
            ]
            missing_factors_names = [
                factor_name[factor]
                for factor in options.keys()
                if factor in input_parameters and not input_parameters[factor]
            ]

            if missing_factors:
                explanation += (
                    f"The following risk factor(s) are missing from the "
                    f"patient's data: {', '.join(missing_factors_names)}. "
                    f"We will assume that these are all absent from the "
                    f"patient. "
                )

            for factor in missing_factors:
                input_parameters[factor] = False

            factors = present_factors + missing_factors + present_but_false

        elif param in ['history', 'electrocardiogram', 'initial_troponin']:
            if param == 'initial_troponin':
                param_name = 'initial troponin'
            else:
                param_name = param

            if param not in input_parameters:
                explanation += (
                    f"'{param_name}' is missing from the patient's data and "
                    f"so we assume it's value is {default_value[param]}."
                )
                input_parameters[param] = default_value[param]
            else:
                explanation += (
                    f"The value of '{param_name}' in the "
                    f"patient's note is determined to be '"
                    f"{param_value}'. "
                )

        elif param == "age":
            age_explanation, age = age_conversion_explanation(
                input_parameters["age"]
            )
            explanation += age_explanation

        # Add points based on parameter value
        if param == 'risk_factors':
            # Compute the number of risk factors
            risk_factors_count = sum(
                1 for factor in factors if input_parameters[factor]
            )
            explanation += (
                f"Based on the HEART Score risk factors criteria, "
                f"{risk_factors_count} risk factors are present and so "
            )

            if risk_factors_count == 0:
                explanation += (
                    f"0 points are added for the risk factors criteria, "
                    f"keeping the current total at {total_score}.\n"
                )
            elif 1 <= risk_factors_count <= 2:
                explanation += (
                    f"1 point is added for the risk factors criteria, "
                    f"making the current total, {total_score} + 1 = "
                    f"{total_score + 1}.\n"
                )
                total_score += 1
            elif (
                risk_factors_count < 3
                and input_parameters['atherosclerotic_disease']
            ):
                explanation += (
                    f"2 points are added for the risk factors criteria as "
                    f"atherosclerotic disease is present,"
                    f" making the current total {total_score} + 2 = "
                    f"{total_score + 2}.\n"
                )
                total_score += 2
            elif risk_factors_count >= 3:
                explanation += (
                    f"2 points are added as 3 or more risk factors are "
                    f"present, making the current total {total_score} + 2 = "
                    f"{total_score + 2}.\n"
                )
                total_score += 2

        elif param == "age":
            if age < 45:
                explanation += (
                    f"The patient's age is less than 45 years "
                    f"and so keep the current total at "
                    f"{total_score}.\n"
                )
            elif 45 <= age < 65:
                explanation += (
                    f"The patient's age is between 45 and 65 years "
                    f"and so we increment the current total by 1, "
                    f"making the current total {total_score} + 1 = "
                    f"{total_score + 1}.\n"
                )
                total_score += 1
            else:
                explanation += (
                    f"The patient's age is greater than 65 years "
                    f"and so we increment the current total by 2, "
                    f"making the current total {total_score} + 2 = "
                    f"{total_score + 2}.\n"
                )
                total_score += 2
        else:
            points = options[input_parameters[param]]

            if param == 'initial_troponin':
                param = 'initial troponin'

            if points == 0:
                explanation += (
                    f"Based on the HEART Score criteria, 0 points are added "
                    f"for '{param}', keeping the current total at "
                    f"{total_score}.\n"
                )
            elif points == 1:
                explanation += (
                    f"Based on the HEART Score criteria, 1 point is added "
                    f"for '{param}', increasing the current total to"
                    f" {total_score} + 1 = {total_score + 1}.\n"
                )
                total_score += 1
            else:
                explanation += (
                    f"Based on the HEART Score criteria, 2 points are added "
                    f"for '{param}', increasing the current total to"
                    f" {total_score} + 2 = {total_score + 2}.\n"
                )
                total_score += 2

    explanation += (
        f"Based on the patient's data, the HEART Score is {total_score}.\n"
    )

    return {"Explanation": explanation, "Answer": total_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'age': [25, 'years'],
            'hypertension': False,
            'history': 'Slightly suspicious',
            'electrocardiogram': 'Normal',
            'diabetes_mellitus': False,
            'smoking': False,
            'atherosclerotic_disease': False,
            'hypercholesterolemia': False,
            'obesity': False,
            'family_with_cvd': False,
            'initial_troponin': 'less than or equal to normal limit',
            'risk_factors': {},
        },
        {
            'age': [57, 'years'],
            'history': 'Moderately suspicious',
            'electrocardiogram': 'Normal',
            'hypertension': True,
            'hypercholesterolemia': True,
            'diabetes_mellitus': False,
            'obesity': False,
            'smoking': False,
            'family_with_cvd': False,
            'atherosclerotic_disease': False,
            'initial_troponin': 'less than or equal to normal limit',
            'risk_factors': {'hypertension', 'hypercholesterolemia'},
        },
        {
            'age': [60, 'years'],
            'hypertension': False,
            'history': 'Slightly suspicious',
            'diabetes_mellitus': False,
            'smoking': False,
            'family_with_cvd': True,
            'atherosclerotic_disease': False,
            'initial_troponin': 'less than or equal to normal limit',
            'electrocardiogram': 'Normal',
            'hypercholesterolemia': False,
            'obesity': False,
            'risk_factors': {'family_with_cvd'},
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_heart_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from
    the 'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function vol_to_vol_explanation
- translation

Date: March 2025
"""

from .rounding import round_number


def vol_to_vol_explanation(
    value, src_unit, tgt_unit, compound="", conversion_factor=False
):
    conversion_factors_L = {
        'L': 1,
        'dL': 0.1,
        'mL': 0.001,
        'µL': 0.000001,
        'mm^3': 0.000001,
        'cm^3': 0.001,
        'm^3': 1000,
    }

    explanation = ""

    if src_unit == tgt_unit:
        return f"The volume of {compound} is {value} {tgt_unit}. ", value
    else:
        if not conversion_factor:
            answer = round_number(value * conversion_factor)

            explanation += (
                f"To convert {src_unit} {compound} to {tgt_unit}, "
                f"multiply by the conversion factor {conversion_factor} "
                f"{tgt_unit}/{src_unit}, resulting to {src_unit} {compound} "
                f"* {conversion_factor} {tgt_unit}/{src_unit} = "
                f"{answer} {tgt_unit} {compound}. "
            )
        else:
            conversion_factor = (
                conversion_factors_L[src_unit] / conversion_factors_L[tgt_unit]
            )
            answer = round_number(conversion_factor)
            explanation += (
                f"The conversion factor is {answer} {tgt_unit} "
                f"for every unit of {src_unit}. "
            )

    return explanation, answer


def molg_to_molg_explanation(mass, compound, src_unit, tgt_unit):
    conversion_factors_mol = {
        'mol': 1,
        'mmol': 0.001,
        'µmol': 0.000001,
        'pmol': 0.000000001,
    }

    conversion_factors_g = {'kg': 1000, 'g': 1, 'mg': 0.001, 'µg': 0.000001}

    conversion_factors_dict = {}

    if "mol" in src_unit and "mol" in tgt_unit:
        conversion_factors_dict = conversion_factors_mol
    elif "g" in src_unit and "g" in tgt_unit:
        conversion_factors_dict = conversion_factors_g

    explanation = ""

    if src_unit == tgt_unit:
        return f"The mass of {compound} is {mass} {tgt_unit}. ", mass
    else:
        conversion_factor = round_number(
            conversion_factors_dict[src_unit]
            / conversion_factors_dict[tgt_unit]
        )

        answer = round_number(mass * conversion_factor)

        explanation += (
            f"To convert {mass} {src_unit} of {compound} to {tgt_unit}, "
            f"multiply by the conversion factor {conversion_factor}, "
            f"giving us {mass} {src_unit} {compound} * {conversion_factor} "
            f"{tgt_unit}/{src_unit} = {answer} {tgt_unit} {compound}. "
        )

        return explanation, answer


def mol_g_explanation(value, compound, molar_mass, src_unit, tgt_unit):
    explanation = ""

    mol_explanation, mol = molg_to_molg_explanation(
        value, compound, src_unit, 'mol'
    )

    explanation += mol_explanation

    grams_value = round_number(mol * molar_mass)

    explanation += (
        f"To convert from mol {compound} to grams, multiply by the molar "
        f"mass {molar_mass} g/mol, which will give {mol} mol {compound} * "
        f"{molar_mass} g/mol = {grams_value} g {compound}. "
    )

    grams_explanation, grams = molg_to_molg_explanation(
        grams_value, compound, "g", tgt_unit
    )

    explanation += grams_explanation

    return explanation, grams


def g_to_mol_explanation(value, compound, molar_mass, src_unit, tgt_unit):
    explanation = ""

    g_explanation, grams = molg_to_molg_explanation(
        value, compound, src_unit, 'g'
    )

    explanation += g_explanation

    mol = round_number(grams / molar_mass)

    explanation += (
        f"To convert from grams of {compound} to moles, divide by the molar "
        f"mass {molar_mass} g/mol, which will give {grams} "
        f"g/({molar_mass} g {compound}/mol) = {mol} mol {compound}. "
    )

    mol_explanation, answer = molg_to_molg_explanation(
        mol, compound, 'mol', tgt_unit
    )

    explanation += mol_explanation

    return explanation, answer


def mEq_to_mol_explanation(value, compound, valence, tgt_unit):
    explanation = (
        f"To convert from {value} mEq to {tgt_unit}, convert from "
        f"mEq to mmol. "
    )

    mol = round_number(value / valence)

    explanation += (
        f"The compound {value} has a valence of {valence}, "
        f"and so divide the valence by the value of mEq to get, "
        f"{value} mEq/({valence} mEq/mmol) = {mol} mmol {compound}. "
    )

    if tgt_unit != 'mmol':
        mol_explanation, mol = molg_to_molg_explanation(
            mol, compound, 'mmol', tgt_unit
        )

        explanation += mol_explanation

    return explanation, mol


def mol_to_mEq_explanation(value, compound, valence, src_unit):
    mmol_value = value

    explanation = ""

    if src_unit != 'mmol':
        explanation = (
            f"To convert from {value} {src_unit} to mEq, "
            f"first convert from {src_unit} to mmol. "
        )
        mmol_explanation, mmol_value = molg_to_molg_explanation(
            value, compound, src_unit, 'mmol'
        )
        explanation += mmol_explanation

    mEq_val = round_number(mmol_value * valence)

    explanation += (
        f"The compound, {compound}, has a valence of {valence}, "
        f"and so multiply the valence by the value of mmol to get, {value} "
        f"mmol * {valence} mEq/mmol = {mEq_val} mEq {compound}. "
    )

    return explanation, mEq_val


def mEq_to_g_explanation(value, compound, molar_mass, valence, tgt_unit):
    explanation = (
        f"To convert from {value} mEq to {tgt_unit} mmol, "
        f"first convert from mEq to mmol. "
    )

    mmol_val = round_number(value / valence)

    explanation += (
        f"The compound, {compound}, has a valence of {valence}, and so "
        f"divide the valence by the value of mEq to get, {value} mEq/("
        f"{valence} mEq/mmol) = {mmol_val} mmol {compound}. "
    )

    mol_explanation, mol_value = molg_to_molg_explanation(
        mmol_val, compound, 'mmol', 'mol'
    )

    explanation += mol_explanation

    gram_explanation, answer = mol_g_explanation(
        mol_value, compound, molar_mass, 'mol', tgt_unit
    )

    explanation += gram_explanation

    return explanation, answer


def g_to_mEq_explanation(value, compound, molar_mass, valence, src_unit):
    explanation = (
        f"To convert from {value} {src_unit} to "
        f"mEq, first convert from {src_unit} to mmol."
    )

    mol_explanation, mol_value = g_to_mol_explanation(
        value, compound, molar_mass, src_unit, "mmol"
    )

    explanation += mol_explanation

    answer = round_number(mol_value * valence)

    explanation += (
        f"To convert from {mol_value} mmol {compound} to mEq, "
        f"multiply the mmol amount by the valence, "
        f"to get {mol_value} mmol * {valence} mEq/mmol = {answer} mEq "
        f"{compound}. "
    )

    return explanation, answer


def conversion_explanation(
    value, compound, molar_mass, valence, src_unit, tgt_unit
):
    conversion_factors_mass = {
        'mol',
        'mmol',
        'µmol',
        'pmol',
        'kg',
        'g',
        'mg',
        'µg',
        'mEq',
    }
    conversion_factors_volume = {'L', 'dL', 'mL', 'µL', 'mm^3', 'cm^3', 'm^3'}

    explanation = ""

    if "/" in src_unit and "/" in tgt_unit:
        src_mass_unit = src_unit.split("/")[0]
        src_volume_unit = src_unit.split("/")[1]

        tgt_mass_unit = tgt_unit.split("/")[0]
        tgt_volume_unit = tgt_unit.split("/")[1]

        if (
            src_mass_unit == tgt_mass_unit
            and src_volume_unit == tgt_volume_unit
        ):
            return (
                f"The concentration of {compound} is {value} "
                f"{src_mass_unit}/{src_volume_unit}. ",
                value,
            )

        explanation += (
            f"The concentration of {compound} is {value} {src_mass_unit}/"
            f"{src_volume_unit}. We need to convert the concentration to "
            f"{tgt_mass_unit}/{tgt_volume_unit}. "
        )

        if src_mass_unit != tgt_mass_unit:
            explanation += (
                f"Let's first convert the mass of {compound} "
                f"from {src_mass_unit} to {tgt_mass_unit}. "
            )
            mass_explanation, mass_value = mass_conversion_explanation(
                value,
                compound,
                valence,
                molar_mass,
                src_mass_unit,
                tgt_mass_unit,
            )
        else:
            explanation += (
                "The mass units of the source and target are the "
                "same so no conversion is needed. "
            )
            mass_value = value

        if src_mass_unit != tgt_mass_unit:
            explanation += mass_explanation

        if src_volume_unit == tgt_volume_unit:
            volume_conversion_factor = 1
            result = round_number(mass_value / volume_conversion_factor)

            explanation += (
                f"The volume units is {tgt_volume_unit} so no "
                f"volume conversion is needed. "
            )
            explanation += (
                f"Hence, the concentration value of {value} {src_mass_unit} "
                f"{compound}/{src_volume_unit} converts to {result} "
                f"{tgt_mass_unit} {compound}/{tgt_volume_unit}. "
            )

            return explanation, result
        else:
            explanation += (
                f"The current volume unit is {src_volume_unit} "
                f"and the target volume unit is {tgt_volume_unit}. "
            )
            explanation_volume, volume_conversion_factor = (
                vol_to_vol_explanation(
                    1, src_volume_unit, tgt_volume_unit, "water", True
                )
            )
            explanation += explanation_volume

        result = round_number(mass_value / volume_conversion_factor)

        explanation += (
            f"Our next step will be to divide the mass by the volume "
            f"conversion factor of {volume_conversion_factor} "
            f"to get the final concentration in terms of {tgt_mass_unit}/"
            f"{tgt_volume_unit}. "
        )
        explanation += (
            f"This will result to {mass_value} {tgt_mass_unit} "
            f"{compound}/{volume_conversion_factor} "
            f"{tgt_volume_unit} = {result} {tgt_mass_unit} "
            f"{compound}/{tgt_volume_unit}. "
        )
        explanation += (
            f"The concentration value of {value} {src_mass_unit} "
            f"{compound}/{src_volume_unit} converts to {result} "
            f"{tgt_mass_unit} {compound}/{tgt_volume_unit}. "
        )
    elif (
        ("/" not in src_unit and "/" not in tgt_unit)
        and src_unit in conversion_factors_mass
        and tgt_unit in conversion_factors_mass
    ):
        if src_unit == tgt_unit:
            return (
                f"The mass of {compound} is {round_number(value)} "
                f"{tgt_unit}. ",
                value,
            )

        explanation, result = mass_conversion_explanation(
            value, compound, valence, molar_mass, src_unit, tgt_unit
        )

    elif (
        ("/" not in src_unit and "/" not in tgt_unit)
        and src_unit in conversion_factors_volume
        and tgt_unit in conversion_factors_volume
    ):
        if src_unit == tgt_unit:
            return f"The volume is {value} {tgt_unit}. ", value

        explanation, value = vol_to_vol_explanation(value, src_unit, tgt_unit)

    return explanation, result


def mass_conversion_explanation(
    value, compound, valence, molar_mass, src_mass_unit, tgt_mass_unit
):
    explanation = f"The mass of {compound} is {value} {src_mass_unit}. "

    if ("g" in src_mass_unit and "g" in tgt_mass_unit) or (
        "mol" in src_mass_unit and "mol" in tgt_mass_unit
    ):
        conv_explanation, mass_value = molg_to_molg_explanation(
            value, compound, src_mass_unit, tgt_mass_unit
        )
        explanation += conv_explanation

    elif "mol" in src_mass_unit and "g" in tgt_mass_unit:
        conv_explanation, mass_value = mol_g_explanation(
            value, compound, molar_mass, src_mass_unit, tgt_mass_unit
        )
        explanation += conv_explanation

    elif "g" in src_mass_unit and "mol" in tgt_mass_unit:
        conv_explanation, mass_value = g_to_mol_explanation(
            value, compound, molar_mass, src_mass_unit, tgt_mass_unit
        )
        explanation += conv_explanation

    elif "mol" in src_mass_unit and "mEq" in tgt_mass_unit:
        conv_explanation, mass_value = mol_to_mEq_explanation(
            value, compound, valence, src_mass_unit
        )
        explanation += conv_explanation

    elif "mEq" in src_mass_unit and "mol" in tgt_mass_unit:
        conv_explanation, mass_value = mEq_to_mol_explanation(
            value, compound, valence, tgt_mass_unit
        )
        explanation += conv_explanation

    elif "mEq" in src_mass_unit and "g" in tgt_mass_unit:
        conv_explanation, mass_value = mEq_to_g_explanation(
            value, compound, molar_mass, valence, tgt_mass_unit
        )
        explanation += conv_explanation

    elif "g" in src_mass_unit and "mEq" in tgt_mass_unit:
        conv_explanation, mass_value = g_to_mEq_explanation(
            value, compound, molar_mass, valence, src_mass_unit
        )
        explanation += conv_explanation

    return explanation, mass_value


def convert_to_units_per_liter_explanation(value, unit, compound, target_unit):
    # Dictionary to define conversion factors to liters
    unit_to_liter = {
        'L': 1,
        'dL': 1e-1,
        'mL': 1e-3,
        'µL': 1e-6,
        'mm^3': 1e-6,
        'cm^3': 1e-3,
        'm^3': 1e3,
    }

    explanation = ""

    if unit == target_unit:
        return (
            f"The patient's concentration of {compound} is {value} count/"
            f"{unit}. ",
            value,
        )
    else:
        explanation += (
            f"The patient's concentration of {compound} is "
            f"{value} count/{unit}. "
        )

    conversions_factor = round_number(
        unit_to_liter[target_unit] / unit_to_liter[unit]
    )
    # print(conversions_factor)

    result = round_number(conversions_factor * value)

    explanation += (
        f"To convert {value} count/{unit} of {compound} to {target_unit}, "
        f"multiply by the conversion factor {conversions_factor} {unit}/"
        f"{target_unit} which will give {value} {compound} count/{unit} * "
        f"{conversions_factor} {unit}/{target_unit} = {result} {compound} "
        f"count/{target_unit}. "
    )
    return explanation, result


def mmHg_to_kPa_explanation(mmHg, compound):
    answer = round_number(0.133322 * mmHg)
    explanation = (
        f"To convert the partial pressure of {compound} from mm Hg of "
        f"{compound} to kPa, multiply by the conversion factor of 0.133322 "
        f"mm Hg/kPa, which will give us {mmHg} * 0.133322 mm Hg/kPa = "
        f"{answer} kPa of {compound}.\n"
    )

    return explanation, answer


def kPa_to_mmHg_explanation(kPa, compound):
    answer = round_number(7.50062 * kPa)
    explanation = (
        f"To convert the partial pressure of {compound} from mm Hg of "
        f"{compound} to kPa, multiply by the conversion factor of 0.133322 "
        f"mm Hg/kPa, which will give us {kPa} * 0.133322 mm Hg/kPa = "
        f"{answer} kPa of {compound}.\n"
    )

    return explanation, answer



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function bmi_calculator_explanation
- translation

Date: March 2025
"""

from .utils.height_conversion import (
    height_conversion_explanation,
)
from .utils.rounding import round_number
from .utils.weight_conversion import (
    weight_conversion_explanation,
)


def bmi_calculator_explanation(input_variables):
    r"""
    Calculates the patient's BMI and generates a detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "weight" (tuple): The patient's weight information in the
            format (value, unit).
                - Value (float): The numerical weight measurement.
                - Unit (str): The unit of weight, which can be "lbs" (
                pounds), "g" (grams), or "kg" (kilograms).
            - "height" (tuple): The patient's height information in the
            format (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height, which can be "cm" (
                centimeters) or "in" (inches).

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's BMI.

    Notes:
        - Uses the `height_conversion_explanation` function to convert
        height to inches.
        - Uses the `weight_conversion_explanation` function to convert
        weight to kilogram.

    Example:
        bmi_calculator_explanation({"weight": (150, "lbs"), "height": (170,
        "cm")})

        output: "{'Explanation': "The formula for computing the patient's
        BMI is (weight)/(height * height), where weight is the patient's
        weight in kg and height is the patient's height in m. \nThe
        patient's height is 170 cm, which is 170 cm * 1 m / 100 cm = 1.7 m.
        The patient's weight is 150 lbs so this converts to 150 lbs *
        0.453592 kg/lbs = 68.039 kg. The patient's bmi is therefore 68.039
        kg / (1.7 m * 1.7 m) = 23.543 kg/m^2.", 'Answer': 23.543}"
    """

    height_explanation, height = height_conversion_explanation(
        input_variables["height"]
    )
    weight_explanation, weight = weight_conversion_explanation(
        input_variables["weight"]
    )

    output = (
        "The formula for computing the patient's BMI is (weight)/(height * "
        "height), where weight is the patient's weight in kg and height is "
        "the patient's height in m.\n"
    )

    output += height_explanation
    output += weight_explanation
    ans = round_number(weight / (height * height))
    output += (
        f"The patient's bmi is therefore {weight} kg / ({height} m * "
        f"{height} m) = {ans} kg/m^2."
    )

    return {"Explanation": output, "Answer": ans}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "weight": (150, "lbs"),  # weight 150 lbs
            "height": (170, "cm"),  # height 170 cm
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = bmi_calculator_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_child_pugh_score_explanation
- translation

Date: March 2025
"""

from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_child_pugh_score_explanation(input_variables):
    r"""
    Calculates the patient's child pugh score and generates a detailed
    explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "inr" (float): The patient's international normalised ratio (
            INR) in the float format.
            - "albumin" (tuple): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): The numerical albumin concentration value.
                - Unit (str): The unit of albumin concentration, eg. "g/L",
                "mg/dL", "g/mL" and so on.
            - "bilirubin" (array): The patient's bilirubin level in the
            format (value, unit).
                - Value (float): The value of bilirubin level.
                - Unit (str): The unit of bilirubin level, eg. "mmol/L",
                "mEq/L", and so on.
            - "ascites" (str): The patient's ascites level.
                - Absent
                - Slight
                - Moderate
            - "encephalopathy" (str): Whether the patient has encephalopathy.
                - No Encephalopathy
                - Grade 1-2
                - Grade 3-4

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's child pugh score.

    Notes:
        - None

    Example:
        compute_child_pugh_score_explanation({'bilirubin': (2.8, 'mg/dL'),
        'albumin': (2.1, 'g/dL'),
        'inr': 1.5,
        'ascites': 'Absent',
        'encephalopathy': 'Grade 1-2'})

        output: "{'Explanation': "\n    The criteria for the Child-PughScore
        are listed below:\n\n    1. Bilirubin (Total): <2 mg/dL (<34.2
        μmol/L) = +1 point, 2-3 mg/dL (34.2-51.3μmol/L) = +2 points,
        \n    >3 mg/dL (>51.3μmol/L) = +3 points\n    2. Albumin: >3.5 g/dL
        (>35 g/L) = +1 point,2.8-3.5 g/dL (28-35 g/L) = +2 points,<2.8 g/dL
        (<28 g/L) = +3 points\n    3. INR: <1.7 = +1 point,1.7-2.3 = +2
        points, >2.3 = +3 points\n    4.Ascites: Absent = +1 point, Slight
        = +2 points, Moderate = +3points\n    5. Encephalopathy:
        NoEncephalopathy = +1 point, Grade 1-2 = +2 points, Grade 3-4 =
        +3points\n\n    The Child-Pugh Score iscalculated by summing the
        points for each criterion.\\n\\n\n    Thecurrent child pugh score is 0.
        \nThepatient's INR is 1.5. Because the INR is less than 1.7, we add
        1 tothe score, making the current total0 + 1 = 1.\nThe concentration
        of bilirubin is 2.8 mg/dL. Because the Bilirubin concentration is
        between 2mg/dL and 3 mg/dL, we add 2 to the score, making the
        current total 1 + 2 = 3.\nThe concentration of albuminis 2.1 g/dL.
        Because the Albumin concentration is less than 2.8 g/dL, we add 3 to
        the score, making thecurrent total 3 + 3 = 6.\nAscites is reported
        to be 'absent' and so we add 1 point to the score, making thecurrent
        total 6 + 1 = 7.\nEncephalopathy state is 'Grade 1-2 encephalopathy'
        and so we add two points to thescore, making the current total 7 + 2 =
        9.\nThe patient's child pugh score is 9.\n", 'Answer': 9}"
    """

    cp_score = 0

    explanation = r"""
    The criteria for the Child-Pugh Score are listed below:

    1. Bilirubin (Total): <2 mg/dL (<34.2 μmol/L) = +1 point, 2-3 mg/dL
        (34.2-51.3 μmol/L) = +2 points, >3 mg/dL (>51.3 μmol/L) = +3 points
    2. Albumin: >3.5 g/dL (>35 g/L) = +1 point, 2.8-3.5 g/dL (28-35 g/L)
        = +2 points, <2.8 g/dL (<28 g/L) = +3 points
    3. INR: <1.7 = +1 point, 1.7-2.3 = +2 points, >2.3 = +3 points
    4. Ascites: Absent = +1 point, Slight = +2 points, Moderate = +3 points
    5. Encephalopathy: No Encephalopathy = +1 point, Grade 1-2 = +2 points,
        Grade 3-4 = +3 points

    The Child-Pugh Score is calculated by summing
    the points for each criterion.\n\n
    """

    explanation += "The current child pugh score is 0.\n"

    inr = input_variables['inr']

    ascites_state = input_variables.get('ascites', 'Absent')
    encephalopathy_state = input_variables.get(
        'encephalopathy', 'No Encephalopathy'
    )

    explanation += f"The patient's INR is {inr}. "
    bilirubin_exp, bilirubin = conversion_explanation(
        input_variables['bilirubin'][0],
        'bilirubin',
        548.66,
        None,
        input_variables['bilirubin'][1],
        "mg/dL",
    )

    albumin_exp, albumin = conversion_explanation(
        input_variables['albumin'][0],
        'albumin',
        66500,
        None,
        input_variables['albumin'][1],
        "g/dL",
    )

    # INR score calculation
    if inr < 1.7:
        explanation += (
            f"Because the INR is less than 1.7, we add 1 to the score, "
            f"making the current total {cp_score} + 1 = {cp_score + 1}.\n"
        )
        cp_score += 1
    elif 1.7 <= inr <= 2.3:
        explanation += (
            f"Because the INR is between 1.7 and 2.3, "
            f"we add two to the score, making the current "
            f"total {cp_score} + 2 = {cp_score + 2}.\n"
        )
        cp_score += 2
    elif inr > 2.3:
        explanation += (
            f"Because the INR is greater than 2.3, we add three to the "
            f"score, making the current total"
            f" {cp_score} + 3 = {cp_score + 3}.\n"
        )
        cp_score += 3

    explanation += bilirubin_exp

    # Bilirubin score calculation
    if bilirubin < 2:
        explanation += (
            f"Because the Bilirubin concentration is less than 2 mg/dL, "
            f"we add 1 to the score, making the "
            f"current total {cp_score} + 1 = {cp_score + 1}.\n"
        )
        cp_score += 1
    elif 2 < bilirubin < 3:
        explanation += (
            f"Because the Bilirubin concentration is between 2 mg/dL and 3 "
            f"mg/dL, we add 2 to the score, "
            f"making the current total {cp_score} + 2 = {cp_score + 2}.\n"
        )
        cp_score += 2
    elif bilirubin >= 3:
        explanation += (
            f"Because the Bilirubin concentration is greater than 3 mg/dL, "
            f"we add 3 to the score, "
            f"making the current total {cp_score} + 3 = {cp_score + 3}.\n"
        )
        cp_score += 3

    explanation += albumin_exp

    # Albumin score calculation
    if albumin > 3.5:
        explanation += (
            f"Because the Albumin concentration is greater than 3.5 g/dL, "
            f"we add 1 to the score, "
            f"making the current total {cp_score} + 1 = {cp_score + 1}.\n"
        )
        cp_score += 1
    elif 2.8 < albumin <= 3.5:
        explanation += (
            f"Because the Albumin concentration is between 2.8 g/dL and 3.5 "
            f"g/dL, we add 2 to the score, "
            f"making the current total {cp_score} + 2 = {cp_score + 2}.\n"
        )
        cp_score += 2
    elif albumin <= 2.8:
        explanation += (
            f"Because the Albumin concentration is less than 2.8 g/dL, "
            f"we add 3 to the score, making the "
            f"current total {cp_score} + 3 = {cp_score + 3}.\n"
        )
        cp_score += 3

    # Ascites score calculation
    if 'ascites' in input_variables:
        if input_variables['ascites'] == 'Absent':
            explanation += (
                f"Ascites is reported to be 'absent' and so we add 1 point "
                f"to the score, making the "
                f"current total {cp_score} + 1 = {cp_score + 1}.\n"
            )
            cp_score += 1
        elif ascites_state == 'Slight':
            explanation += (
                f"Ascites is reported to be 'slight' and so we add 2 points "
                f"to the score, making the "
                f"current total {cp_score} + 2 = {cp_score + 2}.\n"
            )
            cp_score += 2
        elif ascites_state == 'Moderate':
            explanation += (
                f"Ascites is reported to be 'moderate' and so we add 3 "
                f"points to the score, making the "
                f"current total {cp_score} + 3 = {cp_score + 3}.\n"
            )
            cp_score += 3
    else:
        explanation += (
            f"The Ascites state not specified, assuming and so we will "
            f"assume it to be absent. This means "
            f"we add 1 point to the score, making the current total"
            f" {cp_score} + 1 = {cp_score + 1}.\n"
        )
        cp_score += 1

    if 'encephalopathy' in input_variables:
        # Encephalopathy score calculation
        if encephalopathy_state == 'No Encephalopathy':
            explanation += (
                f"Encephalopathy state is reported to be "
                f"'no encephalopathy' and so we add one point to "
                f"the score, making the current total {cp_score} + 1 = "
                f"{cp_score + 1}.\n"
            )
            cp_score += 1
        elif encephalopathy_state == 'Grade 1-2':
            explanation += (
                f"Encephalopathy state is 'Grade 1-2 encephalopathy' and so "
                f"we add two points to the score, making the current total"
                f" {cp_score} + 2 = {cp_score + 2}.\n"
            )
            cp_score += 2
        elif encephalopathy_state == 'Grade 3-4':
            explanation += (
                f"Encephalopathy state is 'Grade 3-4 encephalopathy' and so "
                f"we add three points to the score, making the current total"
                f" {cp_score} + 3 = {cp_score + 3}.\n"
            )
            cp_score += 3
    else:
        explanation += (
            f"Encephalopathy state is not specified, and so we assume it's "
            f"value to be 'no encephalopathy.' We add one point to the "
            f"score, making the current total {cp_score} + 1 = "
            f"{cp_score + 1}.\n"
        )
        cp_score += 1

    explanation += f"The patient's child pugh score is {cp_score}.\n"

    return {"Explanation": explanation, "Answer": cp_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "bilirubin": (2.8, "mg/dL"),
            "albumin": (2.1, "g/dL"),
            "inr": 1.5,
            "ascites": "Absent",
            "encephalopathy": "Grade 1-2",
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_child_pugh_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_delta_gap_explanation
- translation

Date: March 2025
"""

from .anion_gap import (
    compute_anion_gap_explanation,
)
from .utils.rounding import round_number


def compute_delta_gap_explanation(input_parameters):
    r"""
    Calculates the patient's delta gap and generates a detailed explanatory
    text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "sodium" (tuple): The patient's blood sodium level in the
            format (value, unit).
                - Value (float): The blood sodium level.
                - Unit (str): The unit of blood sodium level.
            - "chloride" (tuple): The patient's chloride level in the
            format (value, unit).
                - Value (float): The value of chloride level.
                - Unit (str): The unit of chloride level, eg. "mmol/L",
                "mEq/L", and so on.
            - "bicarbonate" (tuple): The patient's bicarbonate level in the
            format (value, unit).
                - Value (float): The value of bicarbonate level.
                - Unit (str): The unit of bicarbonate level, eg. "mmol/L",
                "mEq/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's delta gap.

    Notes:
        - Uses the compute_anion_gap_explanation function to compute anion gap.

    Example:
        compute_delta_gap_explanation({"chloride": (100.0, "mEq/L"),
        "bicarbonate": (19.0, "mEq/L"),"sodium": (135.0, "mEq/L")})

        output: "{'Explanation': "To compute the formula of the delta gap,
        the formula is anion gap (in mEq/L) - 12. The first step is to
        compute the patient's anion gap.\nThe formula for computing a
        patient's anion gap is: sodium (mEq/L) - (chloride (mEq/L)+
        bicarbonate (mEq/L)).\nThe concentration of sodium is 135.0 mEq/L.
        \nThe concentration of chloride is 100.0 mEq/L. \nThe concentration
        of bicarbonate is 19.0 mEq/L. \nPlugging in these values into the
        anion gap formula gives us 135.0 mEq/L - (100.0 mEq/L + 19.0 mEq/L)
        = 16.0 mEq/L. Hence, The patient's anion gap is 16.0
        mEq/L.\nPlugging in 16.0 mEq/L for the delta gap formula, we get
        16.0 - 12 = 4.0 mEq/L. Hence, the patient's delta gap is 4.0
        mEq/L.\n", 'Answer': 4.0}"
    """

    explanation = (
        "To compute the formula of the delta gap, the formula is anion gap ("
        "in mEq/L) - 12. The first step is to compute the patient's anion "
        "gap.\n"
    )

    anion_gap_resp = compute_anion_gap_explanation(input_parameters)

    explanation += anion_gap_resp["Explanation"]

    anion_gap_val = anion_gap_resp["Answer"]

    answer = round_number(anion_gap_val - 12.0)

    explanation += (
        f"Plugging in {anion_gap_val} mEq/L for the delta gap "
        f"formula, we get {anion_gap_val} - 12 = {answer} mEq/L. "
    )
    explanation += f"Hence, the patient's delta gap is {answer} mEq/L.\n"

    return {"Explanation": explanation, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "chloride": (100.0, "mEq/L"),
            "bicarbonate": (19.0, "mEq/L"),
            "sodium": (135.0, "mEq/L"),
        },
        {
            "chloride": (102.0, "mEq/L"),
            "bicarbonate": (22.0, "mEq/L"),
            "sodium": (137.0, "mEq/L"),
        },
    ]

    # {'Chloride': [100.0, 'mEq/L'], 'Bicarbonate': [19.0, 'mEq/L'],
    # 'Sodium': [135.0, 'mEq/L']}

    # {'Chloride': [102.0, 'mEq/L'], 'Bicarbonate': [22.0, 'mEq/L'],
    # 'Sodium': [137.0, 'mEq/L']}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_delta_gap_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function bsa_calculator_explaination
- translation

Date: March 2025
"""

import math

from .utils.height_conversion import (
    height_conversion_explanation_cm,
)
from .utils.rounding import round_number
from .utils.weight_conversion import (
    weight_conversion_explanation,
)


def bsa_calculator_explaination(input_variables):
    r"""
    Calculates the patient's Body Surface Area and generates a detailed
    explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "weight" (tuple): The patient's weight information in the
            format (value, unit).
                - Value (float): The numerical weight measurement.
                - Unit (str): The unit of weight, which can be "lbs" (
                pounds), "g" (grams), or "kg" (kilograms).
            - "height" (tuple): The patient's height information in the
            format (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height, which can be "cm" (
                centimeters) or "in" (inches).

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Body Surface Area.

    Notes:
        - Uses the `height_conversion_explanation_cm` function to convert
        height to cm.
        - Uses the `weight_conversion_explanation` function to convert
        weight to kilogram.

    Example:
        bsa_calculator_explaination({'weight': [58.0, 'kg'], 'height': [
        179.0, 'cm']})

        output: "{'Explanation': "For the body surface area computation,
        the formula is sqrt((weight (in kgs) * height (in cm))/3600,
        where the units of weight is in kg and the units of height is in
        cm.\nThe patient's height is 179.0 cm. \nThe patient's weight is
        58.0 kg. \nTherefore, the patient's bsa is sqrt((58.0 (in kgs) *
        179.0 (in cm))/3600) = 1.698 m^2.", 'Answer': 1.698}"
    """

    height_explaination, height = height_conversion_explanation_cm(
        input_variables["height"]
    )
    weight_explanation, weight = weight_conversion_explanation(
        input_variables["weight"]
    )

    output = (
        "For the body surface area computation, the formula is sqrt((weight "
        "(in kgs) * height (in cm))/3600, where the units of weight is "
        "in kg and the units of height is in cm.\n"
    )

    output += height_explaination + "\n"
    output += weight_explanation + "\n"

    answer = round_number(math.sqrt(weight * height / 3600))
    output += (
        f"Therefore, the patient's bsa is sqrt(({weight} (in kgs) * "
        f"{height} (in cm))/3600) = {answer} m^2."
    )

    return {"Explanation": output, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "weight": (58.0, "kg"),
            "height": (179.0, "cm"),
        }
    ]

    # {'weight': [58.0, 'kg'], 'height': [179.0, 'cm']}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = bsa_calculator_explaination(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function free_water_deficit_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)
from .utils.weight_conversion import (
    weight_conversion_explanation,
)


def free_water_deficit_explanation(input_variables):
    r"""
    Calculates the patient's LDL cholestrol concentration and generates a
    detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "sex" (str): The patient's gender, which can be either "Male"
            or "Female".
            - "age" (array): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".
            - "weight" (tuple): The patient's weight information in the
            format (value, unit).
                - Value (float): The numerical weight measurement.
                - Unit (str): The unit of weight, which can be "lbs" (
                pounds), "g" (grams), or "kg" (kilograms).
            - "sodium" (array): The patient's bicarbonate level in the
            format (value, unit).
                - Value (float): The value of bicarbonate level.
                - Unit (str): The unit of bicarbonate level, eg. "mmol/L",
                "mEq/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's LDL cholestrol concentration.

    Notes:
        - None

    Example:
        free_water_deficit_explanation({'sex': 'Female',
        'age': [17, 'years'],
        'weight': [63.0, 'kg'],
        'sodium': [141.0, 'mEq/L']})

        output: "{'Explanation': "The formula for computing the free water
        deficit is (total body water percentage) * (weight) * (sodium/140 -
        1), where the total body water percentage is a percentage expressed
        as a decimal, weight is in kg, and the sodium concentration is in
        mmol/L.\nThe patient's total body weight percentage is based on the
        patient's age and gender.\nThe patient is 17 years old. The
        patient's is a Female.\nThe patient is less than 18 years old and
        so the patient is a child. This means total body water percentage
        value is 0.6.\nThe patient's weight is 63.0 kg. The concentration
        of sodium is 141.0 mEq/L. We need to convert the concentration to
        mmol/L. Let's first convert the mass of sodium from mEq to mmol.
        The mass of sodium is 141.0 mEq. To convert from 141.0 mEq to mmol,
        convert from mEq to mmol. The compound 141.0 has a valence of 1,
        and so divide the valence by the value of mEq to get, 141.0 mEq/(1
        mEq/mmol) = 141.0 mmol sodium. The volume units is L so no volume
        conversion is needed. Hence, the concentration value of 141.0 mEq
        sodium/L converts to 141.0 mmol sodium/L. Plugging in these values
        into the equation, we get 0.6 * 63.0 * (141.0/140 - 1) = 0.27 L.
        The patient's free body water deficit is 0.27 L.\n",
        'Answer': 0.27}"
    """

    explanation = ""

    explanation += (
        "The formula for computing the free water deficit is "
        "(total body water percentage) * (weight) * (sodium/140 - 1), "
        "where the total body water percentage is a percentage expressed as "
        "a decimal, weight is in kg, and the sodium concentration is in "
        "mmol/L.\n"
    )

    age_exp, age = age_conversion_explanation(input_variables["age"])
    gender = input_variables["sex"]

    explanation += (
        "The patient's total body weight percentage is based on "
        "the patient's age and gender.\n"
    )
    explanation += age_exp
    explanation += f"The patient's is a {gender}.\n"

    if 0 <= age < 18:
        tbw = 0.6
        explanation += (
            "The patient is less than 18 years old and so the patient is a "
            "child. This means total body water percentage value is 0.6.\n"
        )
    elif 18 <= age < 65 and gender == "Male":
        tbw = 0.6
        explanation += (
            "The patient's age is between 18 and 64 and "
            "so the patient is an adult. For adult male's the total body "
            "water percentage value is 0.60.\n"
        )
    elif 18 <= age < 65 and gender == "Female":
        tbw = 0.5
        explanation += (
            "The patient's age is between 18 and 64 and "
            "so the patient is an adult. For adult female's the total body "
            "water percentage value is 0.50.\n"
        )
    elif age >= 65 and gender == "Male":
        tbw = 0.5
        explanation += (
            "The patient's age is greater than 64 years and so the patient "
            "is considered elderly. For elderly male's, "
            "the total body water percentage value is 0.50.\n"
        )
    elif age >= 65 and gender == "Female":
        tbw = 0.45
        explanation += (
            "The patient's age is greater than 64 years and so the patient "
            "is considered elderly. For elderly female's, the total body "
            "water percentage value is 0.45.\n"
        )

    weight_exp, weight = weight_conversion_explanation(
        input_variables["weight"]
    )
    explanation += weight_exp

    sodium_exp, sodium = conversion_explanation(
        input_variables["sodium"][0],
        "sodium",
        22.99,
        1,
        input_variables["sodium"][1],
        "mmol/L",
    )
    explanation += sodium_exp

    answer = round_number(tbw * weight * (sodium / 140 - 1))

    explanation += (
        f"Plugging in these values into the equation, "
        f"we get {tbw} * {weight} * ({sodium}/140 - 1) = {answer} L. "
    )
    explanation += f"The patient's free body water deficit is {answer} L.\n"

    return {"Explanation": explanation, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'sex': 'Female',
            'age': [32, 'years'],
            'weight': [33.0, 'lbs'],
            'sodium': [134.0, 'mmol/L'],
        },
        {
            'sex': 'Female',
            'age': [17, 'years'],
            'weight': [63.0, 'kg'],
            'sodium': [141.0, 'mEq/L'],
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = free_water_deficit_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function ckd_epi_2021
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion,
    age_conversion_explanation,
)
from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def ckd_epi_2021(input_parameters):
    r"""
    Calculates the patient's Glomerular Filtration Rate (GFR) and
    generates a detailed explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "sex" (str): The patient's gender, which can be either "Male"
            or "Female".
            - "creatinine" (tuple): The patient's height information in the
            format (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height,
                which can be "cm" (centimeters) or "in" (inches).
            - "age" (tuple): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Glomerular Filtration Rate (GFR).

    Notes:
        - None

    Example:
        ckd_epi_2021({
            'sex': 'Male',
            'age': (17, 'years'),
            'creatinine': (3.7, 'mg/dL')
        })

        output: "{'Explanation': "The formula for computing GFR is 142 x (
        Scr/A)**B x 0.9938**age x (gender_coeffcient), where the **
        indicates an exponent operation, Scr is the concentration of serum
        creatinine in mg/dL and gender_coefficient is 1.012 if the patient
        is female, else the coeffient is 1. The coefficients A and B are
        dependent on the patient's gender and the patient's creatinine
        concentration.\nThe patient is 17 years old. The patient's gender is
        Male, and so the patient's gender coefficient is 1.0.\nThe
        concentration of Serum Creatinine is 3.7 mg/dL. Because the
        patient's gender is male and the creatinine concentration is
        greater than or equal to 0.9 mg/dL, A = 0.9 and B = -1.2.\nPlugging
        in these values, we get 142 * (3.7/0.9)**-1.2 * 0.9938**17 *
        1.0 = 23.422.\nHence, the GFR value is 23.422 ml/min/1.73 m².\n",
        'Answer': 23.422}"
    """

    age = age_conversion(input_parameters["age"])
    gender = input_parameters["sex"]
    creatinine_val, creatinine_label = (
        input_parameters["creatinine"][0],
        input_parameters["creatinine"][1],
    )

    creatinine_val_exp, creatinine_val = conversion_explanation(
        creatinine_val,
        "creatinine",
        113.12,
        None,
        creatinine_label,
        "mg/dL",
    )

    if creatinine_val <= 0.7 and gender == "Female":
        a = 0.7
        b = -0.241

    elif creatinine_val <= 0.9 and gender == "Male":
        a = 0.9
        b = -0.302

    elif creatinine_val > 0.7 and gender == "Female":
        a = 0.7
        b = -1.2

    elif creatinine_val > 0.9 and gender == "Male":
        a = 0.9
        b = -1.2

    if gender == "Female":
        gender_coefficient = 1.012
    else:
        gender_coefficient = 1

    return 142 * (creatinine_val / a) ** b * 0.9938**age * gender_coefficient


def ckd_epi_2021_explanation(input_parameters):
    explanation = (
        "The formula for computing GFR is 142 x (Scr/A)**B x "
        "0.9938**age x (gender_coeffcient), where the ** indicates "
        "an exponent operation, Scr is the concentration of serum "
        "creatinine in mg/dL and gender_coefficient is 1.012 if "
        "the patient is female, else the coeffient is 1. The "
        "coefficients A and B are dependent on the patient's "
        "gender and the patient's creatinine concentration.\n"
    )

    age_explanation, age = age_conversion_explanation(input_parameters["age"])
    gender = input_parameters["sex"]

    explanation += age_explanation
    explanation += f"The patient's gender is {gender}, "

    if gender == "Female":
        gender_coefficient = 1.012
        explanation += (
            f"and so the patient's gender coefficient is "
            f"{gender_coefficient}.\n"
        )
    else:
        gender_coefficient = 1.000
        explanation += (
            f"and so the patient's gender coefficient is "
            f"{gender_coefficient}.\n"
        )

    creatinine_val, creatinine_label = (
        input_parameters["creatinine"][0],
        input_parameters["creatinine"][1],
    )
    creatinine_val_exp, creatinine_val = conversion_explanation(
        creatinine_val,
        "Serum Creatinine",
        113.12,
        None,
        creatinine_label,
        "mg/dL",
    )

    explanation += creatinine_val_exp

    if creatinine_val <= 0.7 and gender == "Female":
        explanation += (
            "Because the patient's gender is female and the "
            "creatinine concentration is less than or equal to "
            "0.7 mg/dL, A = 0.7 and B = -0.241.\n"
        )
        a = 0.7
        b = -0.241

    elif creatinine_val <= 0.9 and gender == "Male":
        explanation += (
            "Because the patient's gender is male and the "
            "creatinine concentration is less than or equal to "
            "0.9 mg/dL, A = 0.7 and B = -0.302.\n"
        )
        a = 0.7
        b = -0.302

    elif creatinine_val > 0.7 and gender == "Female":
        explanation += (
            "Because the patient's gender is female and the "
            "creatinine concentration is greater than or equal "
            "to 0.7 mg/dL, A = 0.7 and B = -1.2.\n"
        )
        a = 0.7
        b = -1.2

    elif creatinine_val > 0.9 and gender == "Male":
        explanation += (
            "Because the patient's gender is male and the "
            "creatinine concentration is greater than or equal "
            "to 0.9 mg/dL, A = 0.9 and B = -1.2.\n"
        )
        a = 0.9
        b = -1.2

    result = round_number(
        142 * (creatinine_val / a) ** b * 0.9938**age * gender_coefficient
    )

    explanation += (
        f"Plugging in these values, we get 142 * ("
        f"{creatinine_val}/{a})**{b} * {0.9938}**{age} * "
        f"{gender_coefficient} = {result}.\n"
    )
    explanation += f"Hence, the GFR value is {result} ml/min/1.73 m².\n"

    return {"Explanation": explanation, "Answer": result}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {'sex': 'Male', 'age': (17, 'years'), 'creatinine': (3.7, 'mg/dL')}
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = ckd_epi_2021_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_fib4_explanation
- translation

Date: March 2025
"""

import math

from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.rounding import round_number
from .utils.unit_converter_new import (
    convert_to_units_per_liter_explanation,
)


def compute_fib4_explanation(input_parameters):
    r"""
    Calculates the patient's Fibrosis-4 (FIB-4) index and generates a
    detailed explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "age" (tuple): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".
            - "alt" (tuple): The patient's Alanine aminotransferase in the
            format (value, unit).
                - Value (float): The value of Alanine aminotransferase.
                - Unit (str): The unit of Alanine aminotransferase,
                eg. "U/L" and so on.
            - "ast" (tuple): The patient's Aspartate aminotransferase in the
            format (value, unit).
                - Value (float): The value of Aspartate aminotransferase.
                - Unit (str): The unit of Aspartate aminotransferase,
                e.g. "U/L" and so on.
            - "platelet_count" (tuple): The patient's platelet count in the
            format (value, unit).
                - Value (float): The value of platelet count.
                - Unit (str): The unit of platelet count,
                e.g. "µL" and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Fibrosis-4 (FIB-4) index.

    Notes:
        - None

    Example:
        compute_homa_ir_explanation({'age': (17, 'years'),
        'alt': (144.0, 'U/L'),
        'ast': (108.0, 'U/L'),
        'platelet_count': (277000.0, 'µL')})

        output: "{'Explanation': "The formula for computing Fibrosis-4 is
        Fib-4 = (Age * AST) / (Platelet count (in billions) * √ALT),
        where platelet count is the number of billions per L, and the units
        for AST and ALT are both U/L.\nThe patient's concentration of
        AST is 108.0 U/L.\nThe patient's concentration of ALT is 144.0
        U/L.\nThe patient's concentration of platelets is 277000.0 count/µL.
        To convert 277000.0 count/µL of platelets to L,
        multiply by the conversion factor 1000000.0 µL/L which
        will give 277000.0 platelets count/µL * 1000000.0 µL/L =
        277000000000.0 platelets count/L. This means that there
        are 277000000000.0/(10^9) = 277.0 billion platelet counts
        per liter.\nPlugging these values into the formula,
        we get (17 * 108.0)/(277.0 * sqrt(144.0)) = 0.552.
        \nHence, the Fibrosis-4 score is 0.552.", 'Answer': 0.552}"
    """

    explanation = ""

    age_explanation, age = age_conversion_explanation(input_parameters["age"])
    explanation += age_explanation

    ast_value = input_parameters["ast"][0]
    alt_value = input_parameters["alt"][0]
    src_value = input_parameters["platelet_count"][0]
    src_unit = input_parameters["platelet_count"][1]
    explanation = (
        "The formula for computing Fibrosis-4 is Fib-4 = (Age * "
        "AST) / (Platelet count (in billions) * √ALT), "
        "where platelet count is the number of billions per L, "
        "and the units for AST and ALT are both U/L.\n"
    )

    explanation_platelet, platelet_value = (
        convert_to_units_per_liter_explanation(
            src_value, src_unit, "platelets", "L"
        )
    )

    count_platelet_billions = platelet_value / 1e9
    result = round_number(
        (age * ast_value) / (count_platelet_billions * math.sqrt(alt_value))
    )

    explanation += f"The patient's concentration of AST is {ast_value} U/L.\n"
    explanation += f"The patient's concentration of ALT is {alt_value} U/L.\n"

    explanation += (
        f"{explanation_platelet}This means that there are "
        f"{platelet_value}/(10^9) = {count_platelet_billions} "
        f"billion platelet counts per liter.\n"
    )
    explanation += (
        f"Plugging these values into the formula, "
        f"we get ({age} * {ast_value})/"
        f"({count_platelet_billions} * "
        f"sqrt({alt_value})) = {result}.\n"
    )
    explanation += f"Hence, the Fibrosis-4 score is {result}."

    return {"Explanation": explanation, "Answer": result}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "age": (17, 'years'),
            "alt": (144.0, 'U/L'),
            "ast": (108.0, 'U/L'),
            "platelet_count": (277000.0, 'µL'),
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_fib4_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function ibw_explanation
- translation

Date: March 2025
"""

from .utils.height_conversion import height_conversion_explanation_in
from .utils.rounding import round_number


def ibw_explanation(input_variables):
    r"""
    Calculates the patient's Ideal Body Weight (IBW) and generates a
    detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "height" (tuple): The patient's height information in the
            format (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height, which can be "cm" (
                centimeters), "in" (inches), or other supported units.
            - "sex" (str): The patient's gender, which can be either "Male"
            or "Female".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's ideal body weight (in kilograms).

    Notes:
        - Uses the `height_conversion_explanation_in` function to convert
        height to inches.
        - Uses the `round_number` function to round the result.
        - If the input gender is not "Male" or "Female," the function will
        not calculate IBW.

    Example:
        ibw_explanation({'height': (72, 'in'), 'sex': 'Male'})

        output: "{'Explanation': "The patient's gender is Male.\nThe
        patient's height is 170 cm, which is 170 cm * 0.393701 in/cm =
        66.929 in. \nFor males, the ideal body weight (IBW) is calculated
        as follows:\nIBW = 50 kg + 2.3 kg * (height (in inches) -
        60)\nPlugging in the values gives us 50 kg + 2.3 kg * (66.929 (in
        inches) - 60) = 65.937 kg.\nHence, the patient's IBW is 65.937 kg.",
        'Answer': 65.937}"
    """

    height = input_variables["height"]
    gender = input_variables["sex"]

    height_explanation, height = height_conversion_explanation_in(height)

    explanation = (
        f"The patient's gender is {gender}.\n" + f"{height_explanation}\n"
    )
    ibw = 0.0

    if gender == "Male":
        ibw = round_number(50 + 2.3 * (height - 60))
        explanation += (
            f"For males, the ideal body weight (IBW) is calculated as "
            f"follows:\nIBW = 50 kg + 2.3 kg * (height (in inches) - 60)\n"
            f"Plugging in the values gives us 50 kg + 2.3 kg * ({height} "
            f"(in inches) - 60) = {ibw} kg.\n"
        )

    elif gender == "Female":
        ibw = round_number(45.5 + 2.3 * (height - 60))
        explanation += (
            f"For females, the ideal body weight (IBW) is calculated as "
            f"follows:\nIBW = 45.5 kg + 2.3 kg * (height (in inches) - 60)\n"
            f"Plugging in the values gives us 45.5 kg + 2.3 kg * ({height} "
            f"(in inches) - 60) = {ibw} kg.\n"
        )

    explanation += f"Hence, the patient's IBW is {ibw} kg."

    return {"Explanation": explanation, "Answer": ibw}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "height": (72, 'in'),  # weight 150 lbs
            "sex": "Male",  # Male
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = ibw_explanation(input_variables)
        print(result)
        print("-" * 50)
